# coding: utf-8
import pandas as pd
import numpy as np
import time
import tushare as ts

from rrdata.rrdatad.stock.tusharepro import pro
from rrshare.rqFetch.fetch_basic_tusharepro import fetch_delist_stock, fetch_stock_list_tusharepro
from rrshare.rqUtil.rqLogs import rq_util_log_debug, rq_util_log_info, rq_util_log_expection
from rrshare.rqUtil.rqParameter import startDate


def fetch_stock_day_adj_onetradedate_tsro(trade_date=None):
    trade_date = trade_date.replace('-', '') #兼容设置以防日期格式为2001-10-20格式
    lastEx = None
    retry =3
    for _ in range(retry):
        try:
            df_daily=pro.query('daily',trade_date=trade_date)
            df_daily = df_daily.drop(columns='change')
            df_daily['amount'] = (10*df_daily['amount']).apply(lambda x : round(x,2))
            df_daily['avg'] =  (df_daily['amount'] / df_daily['vol']).apply(lambda x : round(x,2))
            #df_daily_basic=pro.query('daily_basic',trade_date=trade_date)
            df_factor=pro.query('adj_factor',trade_date=trade_date)
            break 
        except Exception as ex:
            lastEx = ex
            rq_util_log_info("[{}]TuSharePro数据异常: {}, retrying...".format(trade_date, ex))
        else:
            rq_util_log_info("[{}]TuSharePro异常: {}, retried {} times".format(trade_date,lastEx, retry))
            return None
    
    delist_code = fetch_delist_stock(trade_date)
    #print(delist_code)
    res=pd.merge(df_factor,df_daily,how='left').sort_values(by='ts_code')
    res['symbol']=res['ts_code'].apply(lambda x:x[:6]) #x[7:9].lower()
    res['trade_date'] = pd.to_datetime(res['trade_date'])
    
    res=res[~res['symbol'].isin(delist_code)]
    res.fillna({'pct_chg':0,'vol':0,'amount':0}, inplace=True)
    #print(res)
    return res

   
if __name__ == "__main__":
    import time
    t1 = time.perf_counter()
    print(fetch_stock_day_adj_onetradedate_tsro('2021-01-07'))
    
    t2 = time.perf_counter()
    t = t2 - t1
    print(f"times:  --- {t}")
    




"""
交易时间：：使用——pre + realtime
历史数据更新后： 使用last

功能：计算stock_day_hfq不同时段（periods =[5,20,60,120,250]）的回报率rt和PRS
依赖：stock_day_hfq的日行情->rrshare(pg),stock_day_hfq
数据库： postgresql12
tips: df.round(2)  ;  sql-> "".jion(list), rq_util_get_pre_trade_date ; rq_util_get_last_tradedate
logging replace print
# pd.concat() sort=true -- wrong : update jqdatasdk(envs:jqdata)
"""
import os
import pprint
import subprocess
import time
import datetime
import pickle
import pandas as pd
import numpy as np
from decimal import Decimal
import warnings
warnings.filterwarnings("ignore")
import logging
logging.basicConfig(level=logging.INFO, format=' %(asctime)s- %(levelname)s-%(message)s')

from rrshare.rqUtil import (rq_util_log_debug, rq_util_log_expection, rq_util_log_info)
from rrshare.rqUtil import (rq_util_date_today, trade_date_sse,rq_util_get_trade_range,
                            rq_util_get_last_tradedate, rq_util_get_pre_trade_date)
from rrshare.rqUtil import (PERIODS, is_trade_time_secs_cn, is_tradeday_and_market_opened)
from rrshare.rqUtil import (client_pgsql, read_data_from_pg, read_sql_from_pg)
from rrshare.rqFetch import (stock_code_to_name, fetch_delist_stock, fetch_realtime_price_stock_day_adj)

NList = PERIODS().PERIODS
N1List = list(map(lambda x: x-1,NList))
client_rrshare = client_pgsql('rrshare')
client_rrfactor = client_pgsql('rrfactor')

USERPATH = os.path.expanduser('~')
print(USERPATH)
file_path = f'{USERPATH}/.rrsdk/data'


def read_stock_day_N_to_pickle(table_name='stock_day_adj_fillna', N=265):
    """  read large stockday N>250 need 10mins, cron read data save to pickle for speed"""
    USERPATH = os.path.expanduser('~')
    print(USERPATH)
    file_path = f'{USERPATH}/.rrsdk/data'
    subprocess.run(['mkdir','-p', file_path])
    #subprocess.run(args=["streamlit","run",f"{USERPATH}/rrshare/rrshare/rqWeb/stock_RS_OH_MA_to_streamlit.py"])
    file_path_name = f"{file_path}/{table_name}_{N}.pkl"
    trade_date = rq_util_get_last_tradedate()
    start_date = rq_util_get_pre_trade_date(trade_date,N)
    # TODO if file exists ,read and get lasy max date compare with last tradedate
    # if == pass; else continue
   
    df = read_sql_from_pg(start_date=start_date, data=\
            "trade_date,open,high,low,close,pre_close,vol,amount,\
            adj_factor,code,pct_chg,avg",table_name=table_name, client=client_rrshare)
    delist_code = fetch_delist_stock(trade_date)
    df =df[~df['code'].isin(delist_code)]
    df.to_pickle(file_path_name)
    print(df)
    return df
    
       

def stock_RT_HH_MA_pre(table_name='stock_day_adj_fillna', N=255): # ok
    """caculate i-1 day rt_ma
       prepare to next day  day rs_OH_ma
       迭代， 只算一次， 速度快。
       if stock_RT_HH_MA_pre tabble exist, compare max tradedate and lastTD
       is_new , pass, else pass
    """
    trade_date = rq_util_get_last_tradedate()
    #print(trade_date)
    start_date = rq_util_get_pre_trade_date(trade_date,N)
    '''
    df = read_sql_from_pg(start_date=start_date, data=\
            "trade_date,open,high,low,close,pre_close,vol,amount,\
            adj_factor,code,pct_chg,avg",table_name=table_name, client=client_rrshare)
    #df.to_pickle("~/.rrsdk/data/stock_day_250.pkl")
    '''
    file_path_name = f"{file_path}/{table_name}_{N}.pkl"
    df = pd.read_pickle(file_path_name)
    print(df)
    
    delist_code = fetch_delist_stock(trade_date)
    df =df[~df['code'].isin(delist_code)]
    
    df_i = df.set_index(['trade_date','code'])
    df_i.drop_duplicates(keep='last',inplace=True)
    df_un = df_i.unstack()
    df_un_fill = df_un.fillna(method='ffill')
    vol_mean =  (df_un_fill['vol']*df_un_fill['adj_factor']).rolling(49).mean().unstack()
    vol_chg = 100*((df_un_fill['vol']*df_un_fill['adj_factor']) / \
            (df_un_fill['vol']*df_un_fill['adj_factor']).rolling(49).mean() - 1).unstack()
    df_fill = df_un_fill.stack().reset_index()
    ma=dict()
    rt=dict()
    #rs=dict()
    for i in N1List:
        ma[i] = ((df_un_fill['close']*df_un_fill['adj_factor']).rolling(i).mean()/df_un_fill['adj_factor']).unstack()
        rt[i] = df_un_fill['pct_chg'].rolling(i).sum().unstack()
        #rs[i]= 100*rt[i].unstack(0).rank(axis=1,ascending=True, pct=True).unstack()
    H = (df_un_fill['high']*df_un_fill['adj_factor']).expanding().max()
    HH = (H/df_un_fill['adj_factor']).unstack()
    L = (df_un_fill['low']*df_un_fill['adj_factor']).expanding().min()
    LL = (L/df_un_fill['adj_factor']).unstack()
    
    close = df_un_fill['close'].unstack()
    pct_chg = df_un_fill['pct_chg'].unstack()
    adj_factor = df_un['adj_factor'].unstack()

    pct_ma_rs = pd.DataFrame({
    'close_pre':close, 'pch_chg_pre': pct_chg, 'adj_factor_pre':adj_factor, 'vol_mean':vol_mean, 'vol_chg':vol_chg,
    'ma4':ma[4],'ma9':ma[9],'ma19':ma[19], 'ma59':ma[59],'ma119':ma[119],'ma249':ma[249],
    'rt4':rt[4],'rt9':rt[9],'rt19':rt[19],'rt59':rt[59],'rt119':rt[119], 'rt249': rt[249], 
    'H':HH,'L':LL, 
    })

    data = pct_ma_rs.reset_index()
    data = data[data['trade_date'] == trade_date]
    data.dropna(subset=['adj_factor_pre','close_pre'],inplace=True)
    logging.info(len(data))
    logging.info(f'\n {data.head()}')
 
    save_PRS_to_pg(table_name="stock_RT_HH_MA_pre",data=data,client=client_rrfactor)
    return data 


def stock_RS_OH_MA_last(table_name='stock_day_adj_fillna', N=255): # ok
    """caculate i day rt_hh_ma
       prepare to next day  day rs_OH_ma
       迭代， 只算一次， 速度快。
    """
    trade_date = rq_util_get_last_tradedate()
    #print(trade_date)
    start_date = rq_util_get_pre_trade_date(trade_date,N)
    """
    df = read_sql_from_pg(start_date=start_date, data=\
            "trade_date,open,high,low,close,pre_close,vol,amount,\
            adj_factor,code,pct_chg,avg",table_name=table_name, client=client_rrshare)
    cols = ['open', 'high', 'low', 'close','pre_close','pct_chg', 'vol', 'amount', 'adj_factor','avg']
    for i in cols:
        df[i] = pd.to_numeric(df[i], errors='coerce')
    """
    file_path_name = f"{file_path}/{table_name}_{N}.pkl"
    df = pd.read_pickle(file_path_name)
    print(df)
    
    delist_code = fetch_delist_stock(trade_date) 
    df =df[~df['code'].isin(delist_code)]
    
    df_i = df.set_index(['trade_date','code'])
    df_i.drop_duplicates(keep='last',inplace=True)
    df_un = df_i.unstack()
    df_un_fill = df_un.fillna(method='ffill')
    
    vol_chg = 100*((df_un_fill['vol']*df_un_fill['adj_factor']) / \
            (df_un_fill['vol']*df_un_fill['adj_factor']).rolling(50).mean() - 1).unstack()
    df_fill = df_un_fill.stack().reset_index()
    ma=dict()
    rt=dict()
    rs=dict()
    for i in NList:
        ma[i] = ((df_un_fill['close']*df_un_fill['adj_factor']).rolling(i).mean()/df_un_fill['adj_factor']).unstack()
        rt[i] = df_un_fill['pct_chg'].rolling(i).sum().unstack()
        rs[i]= 100*rt[i].unstack(0).rank(axis=1,ascending=True, pct=True).unstack()
    H = (df_un_fill['high']*df_un_fill['adj_factor']).expanding().max()
    HH = (H/df_un_fill['adj_factor']).unstack()
    L = (df_un_fill['low']*df_un_fill['adj_factor']).expanding().min()
    LL = (L/df_un_fill['adj_factor']).unstack()
    OH = 100*(df_un_fill['close']*df_un_fill['adj_factor']/H ).unstack()
    OL = 100*(df_un_fill['close']*df_un_fill['adj_factor']/L -1).unstack()
    
    close = df_un_fill['close'].unstack()
    pct_chg = df_un_fill['pct_chg'].unstack()
    adj_factor = df_un['adj_factor'].unstack()

    pct_ma_rs = pd.DataFrame({
    'close':close,  'adj_factor':adj_factor,  'pct_chg_pre':pct_chg,
    'vol_chg':vol_chg,
    'ma5':ma[5],'ma10':ma[10],'ma20':ma[20], 'ma60':ma[60],'ma120':ma[120],'ma250':ma[250],
    'rt5':rt[5],'rt10':rt[10],'rt20':rt[20],'rt60':rt[60],'rt120':rt[120], 'rt250': rt[250], 
    'rs5':rt[5],'rs10':rs[10],'rs20':rs[20],'rs60':rs[60],'rs120':rs[120], 'rs250': rt[250],
    'H':HH,'L':LL, 'OH':OH, 'OL':OL,
    })
    
    data = pct_ma_rs.reset_index()
    data.dropna(subset=['adj_factor','close'], inplace=True)
    data = data[data['trade_date'] == trade_date]
    logging.info(len(data))
    logging.info(f'\n {data.head()}')
    #data.to_pickle('/home/rome/.rrshare/data/stock_RT_HH_MA.pkl')
    save_PRS_to_pg(table_name="stock_RS_OH_MA_last",data=data,client=client_rrfactor)
    return data 


def stock_RS_OH_MA():
    df = read_data_from_pg(table_name='stock_RS_OH_MA_last',client=client_rrfactor)
    df.rename(columns={'close_pre':'close','adj_factor_pre':'adj_factor','pct_chg_pre':'pct_chg'}, inplace=True)
    df_name = read_data_from_pg('stock_belong_swl', client_rrshare)[['code', 'name', 'swl_L1', 'swl_L2','swl_L3']]
    df = pd.merge(df,df_name,how='left', on='code')
    df.drop_duplicates(keep='last',inplace=True)
    df.dropna(subset=['name'],inplace=True)
    save_PRS_to_pg(table_name="stock_RS_OH_MA", data=df, client=client_pgsql('rrfactor'))
    logging.info(f'\n {df.head()}')
    return df    
  

def stock_RS_OH_MA_new():
    """ fast and right pkl or h5 or sql
    """
    df = pd.read_sql_table("stock_RT_HH_MA_pre", client_rrfactor)
    #print(df)

    df_last_date =str( max(set(list(df['trade_date'].values))))[0:10]
    print(df_last_date)

    today_ = rq_util_date_today().strftime('%Y-%m-%d') 
    print(is_tradeday_and_market_opened())
    trade_date = today_  if is_tradeday_and_market_opened() else rq_util_get_last_tradedate()
    print(trade_date)
    delist_code = fetch_delist_stock(trade_date) 
    df =df[~df['code'].isin(delist_code)]
    print(trade_date)
    print(trade_date > df_last_date)
      
    if (trade_date > df_last_date): 
        #fetch_realtime_price from rqFetch
        df_p = fetch_realtime_price_stock_day_adj()
        print(df_p)
        df_p = df_p.drop(columns =['ts_code', 'trade_date','amount','open'], axis=1)
        logging.info(df_p.head())
        #concat
        df = pd.merge(df,df_p, how='left', on='code')
        #print(df)
        #vol_chg = 100*((df['vol']*df['adj_factor']) / (df['vol']*df['adj_factor']).rolling(50).mean() - 1)
        col_ma_pre = ['ma4','ma9', 'ma19', 'ma59', 'ma119', 'ma249']
        col_ma = ['ma5','ma10','ma20', 'ma60','ma120','ma250']
        col_rt_pre =['rt4', 'rt9', 'rt19', 'rt59','rt119', 'rt249']
        col_rt =['rt5', 'rt10', 'rt20', 'rt60','rt120', 'rt250']
        col_ma_dict = dict(zip(col_ma,col_ma_pre))
        print(col_ma_dict)
        col_rt_dict = dict(zip(col_rt, col_rt_pre))
        for  k, v in col_ma_dict.items():
            n = int(k[2:])
            #print(n)
            df[k] = ((n-1)*df[v] + df['close']) / n
            #print(df[k])
            df[k] = df[k].apply(lambda x: Decimal(x).quantize(Decimal(('0.00'))))
        for  k, v in col_rt_dict.items():
            n = int(k[2:])
            df[k] = df[v] + df['pct_chg']
        df.dropna(subset=['H','L','name'],inplace=True)   # TODO ???
        #print(df)
        vol_chg = 100*((df['vol']*df['adj_factor']) / \
            (df['vol']*df['adj_factor']).rolling(50).mean() - 1)  # TODO
        df['H'] = df['H']*df['adj_factor_pre'] / df['adj_factor']
        df['H'] = df.apply(lambda x: max(x.H, x.high), axis=1) 
        df['L'] = df['L']*df['adj_factor_pre'] / df['adj_factor']
        df['L'] = df.apply(lambda x: min(x.L, x.low), axis=1)
        df['OH'] = 100*(df['close']/df['H'])
        #df['OH'] = df['OH'].apply(lambda x: Decimal(x).quantize(Decimal((0.00))))
        df['OL'] = 100*(df['close']/df['L'] - 1)
        #df['OL'] = df['OL'].apply(lambda x: Decimal(x).quantize(Decimal((0.00))))

        #rsi= 100*rt[i].unstack(0).rank(axis=1,ascending=True, pct=True).unstack()
        col_rs = ['rs_5','rs_10','rs_20','rs_60','rs_120', 'rs_250']
        col_rs_dict = dict(zip(col_rs,col_rt))
        for k, v in col_rs_dict.items():
            df[k] = 100*df[v].rank(axis=0,ascending=True, pct=True)

        cols = ['date', 'time', 'code', 'close', 'adj_factor', 'pct_chg',\
                'ma5','ma10','ma20', 'ma60', 'ma120','ma250',\
                'rs_5','rs_10','rs_20','rs_60','rs_120', 'rs_250',\
                'rt5', 'rt10', 'rt20', 'rt60','rt120', 'rt250',\
                'H','L','OH','OL']

        cols_old = ['trade_date', 'code','close', 'adj_factor','pct_chg',\
                'ma5','ma10','ma20', 'ma60', 'ma120','ma250',\
                'rs_5','rs_10','rs_20','rs_60','rs_120', 'rs_250',\
                'rt5', 'rt10', 'rt20', 'rt60','rt120', 'rt250',\
                'H','L','OH','OL']
        #print(cols)
        df = df[cols].round(2)

        df_name = read_data_from_pg('stock_belong_swl',client_rrshare)[['code', 'name', 'swl_L1', 'swl_L2','swl_L3']]
        df = pd.merge(df,df_name,how='left', on='code')
        df.drop_duplicates(keep='last',inplace=True)
        df.dropna(subset=['name'],inplace=True)
        df.rename(columns={'date':'trade_date'}, inplace=True)
        save_PRS_to_pg(table_name="stock_RS_OH_MA_new",data=df, client=client_rrfactor)
        logging.info(f"\n {df.head()}")
        return df
    else:
        return read_data_from_pg(table_name='stock_RS_OH_MA',client=client_rrfactor)
        
       
def stock_select_PRS_new():
    df  = pd.read_sql_table('stock_RS_OH_MA_new', client_rrfactor) 
    logging.info(df.head())

    df = df[(df['rs_250'] > 50) & (df['rs_120'] > 70) & (df['rs_60'] > 70) &  (df['rs_20']> 60) \
         & (df['rs_10']> 70) & (df['rs_5']> 70 )]

    df = df[ (df['OH'] > 75)  & (df['OL'] > 20)]

    df = df[(df['ma60'] > df['ma250']) & (df['ma20'] > df['ma250']) & (df['ma10'] > df['ma250'])]

    df =  df[(df['ma20'] > df['ma120']) & (df['ma10'] > df['ma120']) ]

    df = df[['trade_date','time','code','close', 'pct_chg',\
                'ma5','ma10','ma20', 'ma60', 'ma120','ma250',\
                'rs_5','rs_10','rs_20','rs_60','rs_120', 'rs_250',\
                #'rt5', 'rt10', 'rt20', 'rt60','rt120', 'rt250',\
                'H','L','OH','OL',\
                'name', 'swl_L1', 'swl_L2','swl_L3']]
    save_PRS_to_pg('stock_select_PRS', df, client_rrfactor)
    logging.info(f'\n {df}')
    return df
   
def save_PRS_to_pg(table_name="",data="", client=client_rrfactor):
    #写入数据，table_name为表名，‘replace’表示如果同名表存在就替换掉
    if isinstance(data, pd.DataFrame):
        try:
            data.to_sql(table_name, client, index=False, if_exists='replace')
            print(f'写入数据库的表{table_name} , ok')
        except Exception as e:
            print(e)


def update_stock_PRS_day(N=255):
    stock_RT_HH_MA_pre(N=N)
    stock_RS_OH_MA_last(N=N)
    stock_RS_OH_MA()


def update_stock_PRS_new():
    stock_RS_OH_MA_new()
    stock_select_PRS_new()
    

if __name__ == '__main__':

    update_stock_PRS_day(N=255)
    update_stock_PRS_new()
    pass
   
   




# coding=utf-8
import time
import pandas as pd
import easyquotation as eq 
from rrshare.rqUtil import rq_util_log_info
#from apscheduler.schedulers.blocking import BlockinfScheduler
#sched = BlockinfScheduler()

#quotation = eq.use('sina')  # 新浪 ['sina'] 腾讯 ['tencent', 'qq'] 
#获取所有股票行情
#snapshot_all = quotation.market_snapshot(prefix=True) 
# # prefix 参数指定返回的行情字典中的股票代码 key 是否带 sz/sh 前缀
#单只股票
#one_dict  = quotation.real('300146')
# 支持直接指定前缀，如 'sh000001'

def task_time(func_args): # TODO
    def in_func(*args, **kwargs):
        while True:
            start_time=time.time()
            data = func_args(*args, **kwargs)
            if isinstance(data, dict):
                df =  pd.DataFrame(data).T.reset_index()
            rq_util_log_info(time.time() - start_time)
            time.sleep(5)
        return df 
    return in_func
    
    
def fetch_realtime_price_all(df=True,src='sina'): # or ['tecent','qq']
    quotation = eq.use(src)
    price_all = quotation.market_snapshot(prefix=True)
    if not df:
        return price_all
    price_df = pd.DataFrame(price_all).T.reset_index()
    return price_df

def fetch_realtime_price_stock(symbols=None,df=True,src='sina'): # or ['tecent','qq']
    quotation = eq.use(src)
    prices = quotation.stocks(symbols)
    if not df:
        return prices
    return pd.DataFrame(prices).T.reset_index()


def fetch_snapshot_timer():
    #quotation = eq.use('sina')
    while True:
        df = fetch_realtime_price_all()
        print(df[df['index']== 'sz000001'])
        time.sleep(3)
    pass

@task_time
def fetch_snapshot_task():
    df = fetch_realtime_price_all()
    print(df[df['index']== 'sz000001'][['index','name', 'now', 'time']])


#@sched.scheduled_job('cron',hour='9-15',minute='*', second="*/5", day_of_week='mon-fri')
@task_time
def save_to_hdf(df,file_name='/tmp/snapshot_t1.h5'):
    quotation = eq.use('sina')
    price_all = quotation.market_snapshot(prefix=True)
    df = pd.DataFrame(price_all).T.reset_index()
    print(df[df['index']=='sz000001'][['index','name','now','time']])
    df.to_hdf(file_name, key='df', mode='w')
    rq_util_log_info(f'save df to hdf at {file_name}, ok')


def read_from_hdf(file_name='/tmp/snapshot_t1.h5'):
    df = pd.read_hdf(file_name)
    rq_util_log_info(df)
    return df


if __name__ =='__main__':
    
    df = fetch_realtime_price_all()
    print(df)
    print(fetch_realtime_price_stock(['600519','300674','300146', '002236','605338']))
    #fetch_snapshot_timer()
    #fetch_snapshot_task()
    #save_to_hdf(df)
    #read_from_hdf()
    pass




# -*- coding: utf-8 -*-

#init setting
from rrshare.RQSetting.rqLocalize import (cache_path, log_path, rq_path, setting_path, make_dir_path)

# rqUtil code , date, tradedate
from rrshare.rqUtil import (rq_util_if_trade, rq_util_if_tradetime, is_trade_time_secs_cn, rq_util_get_last_tradedate)

# api
from rrshare.rqFetch import pro

# sql-util
from rrshare.rqUtil.rqPgsql import (PgsqlClass, client_pgsql, read_data_from_pg, read_table_from_pg,       
                                    read_sql_from_pg, save_data_to_postgresql)

#rqFactor
from rrshare.rqFactor.stock_RS_OH_MA import update_stock_PRS_day, update_stock_PRS_new
from rrshare.rqFactor.swl_RS_OH_MA import update_swl_PRS_day, update_swl_PRS_new

# record data
from rrshare.rqUpdate.update_stock_day import record_stock_day
from rrshare.rqUpdate.update_swl_day import record_swl_day
from rrshare.rqUpdate.update_stock_RS_OH_MA import record_stock_PRS, record_stock_PRS_new
from rrshare.rqUpdate.update_swl_RS_OH_MA import record_swl_PRS, record_swl_PRS_new

# to streamlit
#from rrshare.rqWeb import main_st  #, main_echart


#record data all
#from rrshare.record_all_data import main_record

#cli
from rrshare.cmds import cli


def _get_version(default='4.2.0'):
    try:
        from pkg_resources import DistributionNotFound, get_distribution
    except ImportError:
        return default
    else:
        try:
            return get_distribution(__package__).version
        except DistributionNotFound:  # Run without install
            return default
        except ValueError:  # Python 3 setup
            return default
        except TypeError:  # Python 2 setup
            return default


__version__ = _get_version()


def entry_point():
    cli()





# coding: utf-8

from tkinter import PAGES
from tracemalloc import start
import requests
import json
import time
import pandas as pd
from rrshare.rqUtil import (rq_util_get_last_tradedate, rq_util_if_trade, rq_util_if_tradetime, is_tradeday_and_market_opened,
                            rq_util_get_trade_range, rq_util_date_today, is_before_tradetime_secs_cn, is_trade_time_secs_cn)
from rrshare.rqUtil.rqParameter import startDate


class Swsindex(object):
    """
    swl1_realtime :
    http://www.swsindex.com/idx0120.aspx?columnid=8832#
    industry class:
    http://www.swsindex.com/downloadfiles.aspx?swindexcode=SwClass&type=530&columnid=8892

    get sw industry index class and realtime quote from swsindex.com by post method.
    swl2 reltime spot: http://www.swsindex.com/idx0120.aspx?columnId=8833
    swl1_daily: http://www.swsindex.com/idx0130.aspx?columnid=8838#
    swl2_daily: http://www.swsindex.com/idx0200.aspx?columnid=8839&type=Day#
    """
    
    def __init__(self):
        
        self.url = 'http://www.swsindex.com/handler.aspx'
        self.headers = {
            'Accept': 'application/json, text/javascript, */*',
            'Accept-Encoding': 'gzip, deflate',
            'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
            'Content-Length': '227',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Cookie': 'ASP.NET_SessionId=l0zdjg2sww1yrfitod5trjuq',
            'Host': 'www.swsindex.com',
            'Origin': 'http://www.swsindex.com',
            'Proxy-Connection': 'keep-alive',  
            'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36',
            'X-Requested-With': 'XMLHttpRequest'
            }
        
        self.swl1 = ('801010','801030','801040','801050','801080','801110','801120','801130','801140', \
                    '801150','801160','801170','801180','801200','801210','801230','801710','801720','801730', \
                '801740','801750','801760','801770','801780','801790','801880','801890','801950','801960','801970','801980')
        
        self.swl2 = ('801011','801012','801014','801015','801016','801017','801018','801019','801032','801033', \
            '801034','801036','801037','801038','801039','801043','801044','801045','801051','801053','801054',\
                '801055','801056','801072','801074','801076','801077','801078','801081','801082','801083','801084', \
            '801085','801086','801092','801093','801095','801096','801101','801102','801103','801104','801111','801112', \
            '801113','801114','801115','801116','801117','801124','801125','801126','801127','801128','801129','801131', \
            '801132','801133','801141','801142','801143','801145','801151','801152','801153','801154','801155','801156', \
            '801161','801163','801178','801179','801181','801183','801191','801193','801194','801202','801203','801204', \
            '801206','801207','801216','801217','801218','801219','801223','801231','801711','801712','801713','801721', \
            '801722','801723','801724','801726','801731','801733','801735','801736','801737','801738','801741','801742', \
            '801743','801744','801745','801764','801765','801766','801767','801768','801769','801782','801783','801784', \
            '801785','801786','801881','801951','801952','801961','801962','801963','801971','801972','801981','801982', \
            '801983','801991','801992','801993','801994','801995')
       
        self.page_swl1_realtime_nums = len(self.swl1) // 20 + 1
        self.page_swl2_realtime_nums = len(self.swl2) // 20 + 1


    def fetch_swsindex_L1_L2_realtime(self, level="L1"):
        """ level = L1 or L2  data from swsindex.com"""
        if level == "L1":
            page_nums = self.page_swl1_realtime_nums
            swl = self.swl1
            self.headers['Referer'] = 'http://www.swsindex.com/idx0120.aspx?columnid=8832'
        if level == "L2":
            page_nums = self.page_swl2_realtime_nums
            swl = self.swl2
            self.headers['Referer'] = 'http://www.swsindex.com/idx0120.aspx?columnid=8833'
        
        result = []
        for i in range(1,page_nums + 1):
            postdata = {
            'tablename': 'swzs',
            'key': 'L1',
            'p': '{}'.format(i),
            'where': f"L1 in{swl}",
            'orderby': '',
            'fieldlist': 'L1,L2,L3,L4,L5,L6,L7,L8,L11',
            'pagecount': f'{len(swl)}',
            'timed': '{}'.format(int(time.time()*1000))
            #其中页码p是变量，一共2页。timed也是变量，通过 time.time() 来获取时间戳然后将取值到千分位
            }
            req = requests.post(self.url, headers=self.headers, data=postdata)
            data = req.content.decode()
            data = data.replace("'",'"')
            data = json.loads(data)['root']
            #print(len(data))
            result.extend(data)
        
        df = pd.DataFrame(result)
        df.columns = ['index', 'name', 'pre_close', 'open', 'amount','high','low','close','vol' ]
        df['name'] = df['name'].apply(lambda x : x.split()[0])
        cols = ['pre_close', 'open', 'amount','high','low','close','vol']
        for i in cols:
            df[i] = pd.to_numeric(df[i], errors='coerce')
            
        df_swl = df.copy()
        df_swl["index_symbol"] = df_swl["index"]
        df_swl["index_code"] = df_swl["index"] + ".SI"
        df_swl["level"] = f"{level}"
        df_swl["name_level"] = df_swl["name"] + "_" + df_swl["level"]
        
        #print(df.dtypes)
        df_swl['change_pct'] = 0.00 if df_swl['close'].values[0] == 0 else 100*(df_swl['close'] / df_swl['pre_close'] -1) 
        date_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        trade_time = date_time   if  is_tradeday_and_market_opened()  else rq_util_get_last_tradedate()
        df_swl['trade_date'] = trade_time
        df_swl = df_swl[['index_symbol', 'name', 'trade_date','pre_close', 'open', 'high','low','close','change_pct', \
            'vol','amount','index_code','name_level','level']]
        return df_swl.round(2)

        
    def fetch_swsindex_L1_L2_daily(self,level="L1", start_date=None, end_date=None,trade_date=None):
        """sw index daily L1 or L2 , just input trade_date or  start_date and end_date,data from swsindex.com  
        """
        if not trade_date:
            if not start_date:
                start_date = startDate
            if not end_date:
                end_date = rq_util_get_last_tradedate()
            
        if trade_date:
            start_date = trade_date
            end_date = trade_date   
        if level == "L1":
            page_nums = self.page_swl1_realtime_nums
            swl = self.swl1
            self.headers['Referer'] = 'http://www.swsindex.com/idx0200.aspx?columnid=8838&typeDay'
        if level == "L2":
            page_nums = self.page_swl1_realtime_nums
            swl = self.swl2
            self.headers['Referer'] = 'http://www.swsindex.com/idx0200.aspx?columnid=8839&typeDay'
              
        PAGES = len(swl) * len(rq_util_get_trade_range(start_date, end_date)) 
        page_nums = PAGES // 20 + 1
        #print(PAGES, page_nums)
        
        result = []
        for i in range(1,page_nums + 1):
            print(f"fecth data from {self.headers['Referer']} pages:  {i} totla pages: {page_nums}")
            postdata = {
            'tablename':'swindexhistory',
            'key': 'id',
            'p': '{}'.format(i),
            'where':  f"  swindexcode in{swl}  and BargainDate>='{start_date}' and  BargainDate<='{end_date}'",
            'orderby': 'swindexcode asc,BargainDate_1', 
            #"SwIndexCode,SwIndexName,BargainDate,CloseIndex,BargainAmount,Markup,TurnoverRate,PE,PB,MeanPrice,BargainSumRate,NegotiablesShareSum,NegotiablesShareSum2,DP",
            'fieldlist': 'SwIndexCode,SwIndexName,BargainDate,CloseIndex,BargainAmount,BargainSum,Markup,OpenIndex,MaxIndex,MinIndex',
            'pagecount': f'{PAGES}',
            'timed': '{}'.format(int(time.time()*1000))
            ##其中页码p是变量，一共2页。timed也是变量，通过 time.time() 来获取时间戳然后将取值到千分位
            }
            #print(postdata)
            req = requests.post(self.url, headers=self.headers, data=postdata)
            data = req.content.decode()
            data = data.replace("'",'"')
            data = json.loads(data)['root']
            #print(len(data))
            result.extend(data)
        df = pd.DataFrame(result)
        
        df.rename(columns={'SwIndexCode':'index_symbol' , 'SwIndexName':'name','BargainDate':'trade_date', 'BargainSum':'vol',
                'BargainAmount':'amount','CloseIndex':'close', 'OpenIndex':'open', 'MaxIndex':'high','MinIndex':'low',
                'Markup':'change_pct'}, inplace=True)
      
        df_swl = df.copy()
        df_swl["index_code"] = df_swl["index_symbol"] + ".SI" 
        
        df_swl["level"] = f"{level}"
        df_swl["name_level"] = df_swl["name"] + "_" + df_swl["level"]
        df_swl['trade_date'] = pd.to_datetime(df_swl['trade_date'],format="%Y%m%d %H:%M:%S")
        df_swl = df_swl[['index_symbol', 'name', 'trade_date', 'open', 'high','low','close','change_pct', \
            'vol','amount','index_code','name_level','level']]
        cols = ['open', 'amount','high','low','close','vol','change_pct']
        for i in cols:
            df_swl[i] = pd.to_numeric(df_swl[i], errors='coerce')
        return df_swl
           
           
    def fetch_swsindex_L1_L2_daily_valucation(self,level="L1", start_date=None, end_date=None,trade_date=None):
        """sw index daily L1 or L2 , just input trade_date or  start_date and end_date,data from swsindex.com """
        if not trade_date:
            if not start_date:
                start_date = startDate
            if not end_date:
                end_date = rq_util_get_last_tradedate()
        if trade_date:
            start_date = trade_date
            end_date = trade_date   
        if level == "L1":
            page_nums = self.page_swl1_realtime_nums
            swl = self.swl1
            self.headers['Referer'] = 'http://www.swsindex.com/idx0200.aspx?columnid=8838&typeDay'
        if level == "L2":
            page_nums = self.page_swl1_realtime_nums
            swl = self.swl2
            self.headers['Referer'] = 'http://www.swsindex.com/idx0200.aspx?columnid=8839&typeDay'
              
        PAGES = len(swl) * len(rq_util_get_trade_range(start_date, end_date)) 
        page_nums = PAGES // 20 + 1
        #print(PAGES, page_nums)
        
        result = []
        for i in range(1, page_nums + 1):
            print(f"fecth data from {self.headers['Referer']} pages:  {i} totla pages: {page_nums}")
            postdata = {
            'tablename':'swindexhistory',
            'key': 'id',
            'p': '{}'.format(i),
            'where':  f"  swindexcode in{swl}  and BargainDate>='{start_date}' and  BargainDate<='{end_date}'",
            'orderby': 'swindexcode asc,BargainDate_1', 
            'fieldlist': "SwIndexCode,SwIndexName,BargainDate,CloseIndex,BargainAmount,Markup,TurnoverRate,PE,PB,MeanPrice,BargainSumRate,NegotiablesShareSum,NegotiablesShareSum2,DP",
            'pagecount': f'{PAGES}',
            'timed': '{}'.format(int(time.time()*1000))
            ##其中页码p是变量，一共2页。timed也是变量，通过 time.time() 来获取时间戳然后将取值到千分位
            }
            #print(postdata)
            req = requests.post(self.url, headers=self.headers, data=postdata)
            data = req.content.decode()
            data = data.replace("'",'"')
            data = json.loads(data)['root']
            #print(len(data))
            result.extend(data)
        
        df = pd.DataFrame(result)
        df.rename(columns={'SwIndexCode':'index_symbol' , 'SwIndexName':'name','BargainDate':'trade_date', 'BargainSum':'vol',
                'BargainAmount':'amount','CloseIndex':'close', 'Markup':'change_pct'}, inplace=True)
        df_swl = df.copy()
        df_swl["index_code"] = df_swl["index_symbol"] + ".SI" 
        df_swl["level"] = f"{level}"
        df_swl["name_level"] = df_swl["name"] + "_" + df_swl["level"]
        df_swl['trade_date'] = pd.to_datetime(df_swl['trade_date'],format="%Y%m%d %H:%M:%S")
        cols = ['amount','close','change_pct']
        for i in cols:
            df_swl[i] = pd.to_numeric(df_swl[i], errors='coerce')
        return df_swl
        
         
    def get_swindex_name(self, out_type=dict):
        data1 = self.fetch_swsindex_L1_L2_realtime("L1")['index','name']
        data2 = self.fetch_swsindex__L1_L2_realtime("L2")[['index', 'name']]
        data = data1.append(data2)
        if out_type == dict:
           
            data.set_index('index', inplace=True)
            return data.to_dict()['name']
        
        return data 
   

    def get_swl1_class_one(self,index='801010'):
        """http://www.swsindex.com/idx0210.aspx?swindexcode=801010
        """
        self.headers['Referer'] = 'http://www.swsindex.com/idx0210.aspx?swindexcode={}'.format(index)
        result = []
        for i in range(1, 100):
            postdata={
            'tablename': 'SwIndexConstituents',
            'key': 'id',
            'p': '{}'.format(i),
            'where': 'SwIndexCode={} and IsReserve =0 and  NewFlag=1'.format(index),
            'orderby': 'StockCode, BeginningDate_0',
            'fieldlist': 'stockcode,stockname',#newweight,beginningdate',
            'pagecount': '92',
            'timed': '{}'.format(int(time.time()*1000))
            }
            #print(postdata)
            
            req = requests.post(self.url, headers=self.headers, data=postdata)
            data = req.content.decode()
            data = data.replace("'",'"')
            data = json.loads(data)['root']
            #print(len(data))
            if not len(data):
                break
            result.extend(data)
        df = pd.DataFrame(result)
        # df.drop(columns=['newweight','beginningdate'], inplace=True)
        df['index_name'] = self.get_index_name(out_type=dict)[index] + '_L1'
        print(df)
        df.rename(columns={'stockcode': 'code','stockname':'name'}, inplace=True)
        return df


    def get_swl1_class(self):
        df_L1 = self.fetch_swsindex_L1_L2_realtime("L1")
        #print(df_L1)
        L1_INDEX = df_L1["index"].values
        
        df = pd.DataFrame()
        for l in L1_INDEX:
            print(f'index code: {l}')
            one = self.get_swl1_class_one(l)
            one['index'] = l
            df = df.append(one)
        return df


def get_sw_index_daily_L1_L2(start_date=None,end_date=None,trade_date=None):
    """ just input tradedate """
    df =pd.DataFrame()
    for l in ["L1","L2"]:
        print(f" sws index level: {l}")
        swl = Swsindex().fetch_swsindex_L1_L2_daily(level=l,start_date=start_date, end_date=end_date, trade_date=trade_date)
        df = df.append(swl)
    #print(df)
    return df


def get_sw_index_daily_valuation_L1_L2(start_date=None,end_date=None,trade_date=None):
    """ just input tradedate """
    df =pd.DataFrame()
    for l in ["L1","L2"]:
        print(f" sws index level: {l}")
        swl = Swsindex().fetch_swsindex_L1_L2_daily_valucation(level=l,start_date=start_date, end_date=end_date, trade_date=trade_date)
        df = df.append(swl)
    #print(df)
    return df

     
if __name__ == '__main__':
    swi = Swsindex()
    print(swi.fetch_swsindex_L1_L2_realtime("L1"))  # ok
    #print(swi.fetch_swsindex_L1_L2_daily(level='L1',trade_date='2022-06-01'))
    #print(swi.fetch_swsindex_L1_L2_daily(level='L1',start_date='2022-01-01', end_date='2022-01-20'))  # TODO
    print(swi.fetch_swsindex_L1_L2_daily_valucation(level='L1',trade_date='2022-06-01'))   
    #print(swi.get_swl1_class_one()) #pk
    #print(swi.get_swl1_class()) #ok
    #print(get_sw_index_daily_L1_L2(trade_date=startDate))
    #print(get_sw_index_daily_L1_L2(start_date="2022-05-02"))
    print(get_sw_index_daily_valuation_L1_L2(start_date='2022-06-01'))
    print(get_sw_index_daily_valuation_L1_L2(trade_date='2022-06-01'))    
 



import re
from typing import TypeVar, Union
from  functools import wraps
import pandas as pd

F = TypeVar('F')

def timer(func):
    """ func taste times(second) decorator
        @functools.wraps : wrapper.__name__ = func.__name__
    """
    def wrapper(*args, **kwargs):
        from time import perf_counter
        time_start = perf_counter()
        result = func(*args, **kwargs)
        time_end = perf_counter()
        spend_time =  time_end - time_start
        print(f" Cost time:  {spend_time:.4f} S")
        return result
    return wrapper
  

def to_numeric(func: F) -> F:
    """ dataframe / series to numeric"""
    ignore = ["ts_code",'index_code','index_symbol','symbol','code',"ID",'name', 'level','name_level']
    
    @wraps(func)
    def wrapper(*args, **kwargs):
        values = func(*args,**kwargs)
        if isinstance(values, pd.DataFrame):
            for column in values.columns:
                if column not in ignore:
                    values[column] = values[column].apply(convert)
        elif isinstance(values, pd.Series):
            for index in values.index:
                if index not in ignore:
                    values[index] =convert(values[index])
        return values
    
    def convert(o:Union[str, int, float]) -> Union[str, int, float]:
        if not re.findall('\d', str(o)):
            return o
        try:
            o = int(o) if str(o).isalnum() else float(o)
        except Exception:
            pass
        return o

    return wrapper
        


# coding: utf-8
import pandas as pd
import numpy as np
import time
import datetime
import tushare as ts
import easyquotation as eq
import warnings
warnings.filterwarnings("ignore")

from rrshare.rqUtil import (rq_util_log_debug, rq_util_log_expection,
                                     rq_util_log_info)
from rrshare.rqUtil import (rq_util_code_tosrccode, rq_util_code_tostr)
from rrshare.rqUtil import rq_util_get_trade_range, is_before_tradetime_secs_cn

from rrshare.rqUtil import (rq_util_date_str2int,rq_util_date_int2str,is_tradeday_and_market_opened,
                            rq_util_date_today, trade_date_sse,is_tradeday_and_market_opened,
                            rq_util_get_last_tradedate)
from rrshare.rqFetch import (pro,fetch_delist_stock, fetch_get_stock_list)
from rrshare.rqUtil import PERIODS
from rrshare.rqUtil.rqParameter import startDate 
PERIODS=PERIODS().PERIODS
print(PERIODS)


def fetch_stock_list_tusharepro():    
    stock_list_l= pro.stock_basic(exchange_id='', is_hs='',list_status='L' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    stock_list_D= pro.stock_basic(exchange_id='', is_hs='',list_status='D' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    stock_list_P= pro.stock_basic(exchange_id='', is_hs='',list_status='P' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')          
    stock_list=pd.concat([stock_list_l,stock_list_D],axis=0)
    stock_list=pd.concat([stock_list,stock_list_P],axis=0)
    return stock_list


def fetch_stock_day_adj_fillna_from_tusharepro(trade_date=startDate):
    trade_date = trade_date.replace('-', '') #兼容设置以防日期格式为2001-10-20格式
    lastEx = None
    retry = 10
    for _ in range(retry):
        try:
            df_daily=pro.query('daily',trade_date=trade_date)
            df_daily = df_daily.drop(columns='change')
            df_daily['amount'] = (10*df_daily['amount']).apply(lambda x : round(x,2))
            df_daily['avg'] =  (df_daily['amount'] / df_daily['vol']).apply(lambda x : round(x,2))
            #df_daily_basic=pro.query('daily_basic',trade_date=trade_date)
            df_factor=pro.query('adj_factor',trade_date=trade_date)             
            break 
        except Exception as ex:
            lastEx = ex
            rq_util_log_info("[{}]TuSharePro数据异常: {}, retrying...".format(trade_date, ex))
        else:
            rq_util_log_info("[{}]TuSharePro异常: {}, retried {} times".format(trade_date,lastEx, retry))
            return None   
    res=pd.merge(df_factor,df_daily,how='left').sort_values(by='ts_code')
    res['code']=res['ts_code'].apply(lambda x:x[:6]) #x[7:9].lower()
    res['trade_date'] = pd.to_datetime(res['trade_date'], format='%Y-%m-%d')
    delist_code = fetch_delist_stock(trade_date)
    res=res[~res['code'].isin(delist_code)]
    res.fillna({'pct_chg':0,'vol':0,'amount':0}, inplace=True)
    print(res)
    return res


def fetch_realtime_price_stock_day_adj(src='sina'):
    """if today is trade date trade_date = today
        else trade_date = last trade_date
        time: 9:30-11:31 , 13:00-15:03
        pro.adj_factor update at 9:30
    """
    #print(is_tradeday_and_market_opened())
    trade_date = rq_util_date_today() if is_tradeday_and_market_opened() else  rq_util_get_last_tradedate()
    #print(trade_date)
    trade_date = str(trade_date).replace('-','')
    quotation = eq.use(src)
    #print(quotation)
    df_factor=pro.query('adj_factor',trade_date=trade_date)
    df_factor['code'] =  df_factor['ts_code'].apply(lambda x: x[0:6])
    all_secs = df_factor['ts_code'].values
    secs_eq = list(map(lambda x: x[0:6], all_secs))
    #print(secs_eq)
    price_all = quotation.stocks(secs_eq)
    #print(price_all)
    df_p = pd.DataFrame(price_all).T.reset_index()
    df_p = df_p[['index','name', 'close','now','open','high','low','turnover','volume','date','time']]
    df_p = df_p.loc[df_p['volume'] > 0]
    df_p['pct_chg'] = (100*(df_p['now']/df_p['close'] - 1)).map(lambda x:round(x,2))
    df_p['avg'] = df_p['volume']/df_p['turnover']
    df_p = df_p.rename(columns={'index':'code','close':'pre_close', 'now':'close','turnover':'vol','volume':'amount'})
    df_p = df_p.sort_values(by='code')
    for i in ['vol', 'amount']:
        df_p[i] = (df_p[i]/100.00).apply(lambda x: round(float(x), 2))
    df = pd.merge(df_factor, df_p, how='left', on='code').sort_values(by = 'code')
    df['trade_date'] = pd.to_datetime(df['trade_date'], format='%Y-%m-%d')
    delist_code = fetch_delist_stock(trade_date)
    df=df[~df['code'].isin(delist_code)]
    
    df.fillna({'pct_chg':0,'vol':0,'amount':0}, inplace=True)
    return df




def fetch_get_stock_day_hfq_one(name, start='', end='', adj='hfq', freq='D', ma=PERIODS,adjfactor=True, type_='pd'):
    """在Tushare数据接口里，不管是旧版的一些接口还是Pro版的行情接口，都是以用户设定的end_date开始往前复权，跟所有行情软件或者财经网站上看到的前复权可能存在差异，
    因为所有这些都是以最近一个交易日开始往前复权的。比如今天是2018年10月26日，您想查2018年1月5日～2018年9月28日的前复权数据，Tushare是先查找9月28日的复权因子，
    从28日开始复权，而行情软件是从10月26日这天开始复权的。
    """
    def fetch_data():
        data = None
        try:
            time.sleep(0.002)
            #pro = get_pro()
            data = ts.pro_bar(
                api=pro,
                ts_code=str(name),
                asset='E',
                adj=adj,
                start_date=start,
                end_date=end,
                ma=ma,
                freq=freq,
                adjfactor=adjfactor,
                factors=['tor',
                         'vr']
            ).sort_index()
            print('fetch done: ' + str(name))
        except Exception as e:
            print(e)
            print('except when fetch data of ' + str(name))
            time.sleep(1)
            data = fetch_data()
        return data
    data = fetch_data()

    #data['date_stamp'] = data['trade_date'].apply(lambda x: cover_time(x))
    data['code'] = data['ts_code'].apply(lambda x: str(x)[0:6])
    data['fqtype'] = adj
    if type_ in ['json']:
        data_json = rq_util_to_json_from_pandas(data)
        return data_json
    elif type_ in ['pd', 'pandas']:
        data['date'] = pd.to_datetime(data['trade_date'], format='%Y%m%d')
        data = data.set_index('date', drop=False)
        data['date'] = data['date'].apply(lambda x: str(x)[0:10])
        return data



if __name__ == '__main__':          
    #print(fetch_stock_list_tusharepro())
    #trade_date = rq_util_get_last_tradedate()
    #df1=fetch_stock_day_adj_fillna_from_tusharepro(trade_date)
    #print(df1)
    df2 = fetch_realtime_price_stock_day_adj()
    print(df2)
    
   
    
   
    
    



from re import I
import pandas as pd 
from rrshare.rqFetch.fetch_swl_index_daily_and_realtime import Swsindex


def sw_index_class(level="L1"):
    """
    swl L1, L2, class
    申万一，二，-分类
    """
    sw_level=f"level{level[-1]}"
    #print(sw_level)
     
    swl_spot = Swsindex().fetch_swsindex_L1_L2_realtime(level)
    print(swl_spot)
    df_swl = swl_spot.copy()
    df_swl["index_code"] = df_swl["index_symbol"] + ".SI" 
  
    df_swl["level"] = f"{level}"
    df_swl["name_level"] = df_swl["name"] + "_" + df_swl["level"]
    df_swl = df_swl[['index_symbol', 'index_code', 'index_symbol', 'level', 'name_level']]

    return df_swl


def sw_index_class_L1_L2():
    df = pd.DataFrame() 
    for level in ["L1", "L2"]:
        df = pd.concat([df,sw_index_class(level)])
    return df


if __name__ == "__main__":
    print(sw_index_class(level="L2"))
    print(sw_index_class("L1").columns)
    
    print(sw_index_class_L1_L2())


# streamlit_echarts
import pandas as pd
from pyecharts.globals import CurrentConfig

from rrshare.rqUtil import (client_pgsql,read_data_from_pg,read_sql_from_pg)
from rrshare.rqUtil import  (rq_util_get_pre_trade_date,rq_util_get_last_tradedate)

conn = client_pgsql('rrfactor')
lastTD = rq_util_get_last_tradedate()
    
cols_prs =  ['code','name',\
         'ma5','ma10','ma20', 'ma60', 'ma120','ma250',\
        'rs_5','rs_10','rs_20','rs_60','rs_120','rs_250',\
            'OH','OL','H','L','swl_L1','swl_L2','swl_L3']

from pyecharts import options as opts
from pyecharts.charts import Bar, Tab, Line, Pie
from pyecharts.components import Table
from streamlit_echarts import st_pyecharts
from streamlit_echarts import st_echarts



def get_swl1_rs():
    df = pd.read_sql_table('swl_rs_L1',conn)
    col = ['name_level','rs_5','rs_10','rs_20','rs_60','rs_120','rs_250']
    df = df[col]
    #print(df)
    return df


def get_swl1_valuation():
    data = pd.read_sql_table('swl_rs_valuation_L1',conn)
    df = data.copy()
    col = ['name_ilevel_x','pe','pb']
    #col1 = col[1:]
    df = df[col]
    return df


def get_etf_rs():#TODO
    data = pd.read_sql_table('etf_rs',conn)
    df = data.copy()
    col = ['name','turnover_rate','rs_3','rs_10','rs_60','rs_120']
    #col1 = col[1:]
    df = df[col]
    return df


def bar_RS() -> Bar:
    df = get_swl1_rs()
    col = df.columns
    x_axis = list(df.name_level.values)
    print(x_axis)
    y_5 =  list(df.rs_5.values)
    y_10 = list(df.rs_10.values)
    y_20 = list(df.rs_20.values)
    y_60 = list(df.rs_60.values)
    y_120 = list(df.rs_120.values)
    y_250 = list(df.rs_250.values)
    print(y_5,y_10,y_20, y_60,y_120, y_250)

    c = (
            Bar()
   #     .set_global_opts(){"width": "4600px", "height":"1960px"})
        .add_xaxis(x_axis)
        .add_yaxis("rs_5", y_5)
        .add_yaxis("rs_10", y_10)
        .add_yaxis("rs_20", y_20)
        .add_yaxis("rs_60", y_60)
        .add_yaxis("rs_120", y_120)
        .add_yaxis("rs_250", y_250)
        .set_global_opts(title_opts=opts.TitleOpts(title="swl1_rs", subtitle=f"{lastTD}" ) ,
            xaxis_opts=opts.AxisOpts(name_rotate=60, name="swl1_name", axislabel_opts={"rotate":45}))
    )
    st_pyecharts(c)


def bar_valucation() -> Bar:
    df = get_swl1_valuation()
    print(df)
    col = df.columns
    print(col)
    x_axis = list(df.name_x.values)
    print(x_axis)
    #y_turnover_ratio =  list(df.turnover_ratio.values)
    y_pe =  list(df.pe.values)
    y_pb =  list(df.pb.values)
    print(y_pe,y_pb)

    c = (
            Bar({"width": "1500px", "height":"560px"})
        .add_xaxis(x_axis)
        #.add_yaxis("turnover_ratio", y_turnover_ratio)
        .add_yaxis("pe", y_pe)
        .add_yaxis("pb", y_pb)

        .set_global_opts(title_opts=opts.TitleOpts(title="swl1-valuation", subtitle=f"{lastTD}" ),
             xaxis_opts=opts.AxisOpts(name_rotate=60, name="swl1_name", axislabel_opts={"rotate":45}) )
    )
    #return c
    st_pyecharts(c)
    return c


def bar_etf_rs() -> Bar:
    df = get_etf_rs()
    print(df)
    col = df.columns
    print(col)
    x_axis = list(df.name.values)
    print(x_axis)
    y_1 = list(df.turnover_rate.values)
    y_3 =  list(df.rs_3.values)
    y_10 = list(df.rs_10.values)
    #y_20 = list(df.rs_20.values)
    y_60 = list(df.rs_60.values)
    y_120 = list(df.rs_120.values)
    print(y_3,y_10, y_60,y_120)

    c = (
            Bar({"width": "1380px", "height":"520px"})
        .add_xaxis(x_axis)
        .add_yaxis("turnover_rate", y_1)
        .add_yaxis("rs_3", y_3)
        .add_yaxis("rs_10", y_10)
       # .add_yaxis("rs_20", y_20)
        .add_yaxis("rs_60", y_60)
        .add_yaxis("rs_120", y_120)
        .set_global_opts(title_opts=opts.TitleOpts(title="etf_rs", subtitle=f"{lastTD}" ) ,
            xaxis_opts=opts.AxisOpts(name_rotate=60, name="etf_name", axislabel_opts={"rotate":45}))
    )
    return c


def st_pyecharts_demo():
    b = (
    Bar()
    .add_xaxis(["Microsoft", "Amazon", "IBM", "Oracle", "Google", "Alibaba"])
    .add_yaxis(
        "2017-2018 Revenue in (billion $)", [21.2, 20.4, 10.3, 6.08, 4, 2.2]
    )
    .set_global_opts(
        title_opts=opts.TitleOpts(
            title="Top cloud providers 2018", subtitle="2017-2018 Revenue"
        ),
        toolbox_opts=opts.ToolboxOpts(),
            )
    )
    st_pyecharts(b)



if __name__ == "__main__":

    #st_pyecharts_demo()
    bar_RS()
    
    pass
    
    



from functools import wraps

class ConSqlDb(object):
    db = []  # 设置连接池
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:#判断单例是否存在,不存在就创建否则直接返回
            cls._instance = object.__new__(cls)
        return cls._instance

class Singleton(object):
    _instance = None
    _is_init = False
    def __new__(cls, *args,**kwargs):
        if not cls._instance:
            cls._instance = super(Singleton,cls).__new__(cls,*args,**kwargs)
        return cls._instance
    """
    def __init__(self,name):
        if not self._is_init:
            self.name = name
            self._is_init = True

    def __str__(self):
        return self.name
    """


def Singleton_wraps(cls):
    _instance = {}
    @wraps(cls)
    def wrapper(*args, **kwargs):
        if cls not in _instance:
            _instance[cls] = cls(*args, **kwargs)
        return _instance[cls]
    return wrapper

if __name__ == '__main__':
    @Singleton_wraps
    class Setting(object):
        def __init__(self):
            pass
    print(id(Setting()),id(Setting()))











import datetime
import os
import sys
from zenlog import logging
from rrshare.RQSetting.rqLocalize import log_path, setting_path


"""2019-01-03  升级到warning级别 不然大量别的代码的log会批量输出出来
2020-02-19 默认使用本地log 不再和数据库同步
"""

try:
    _name = '{}{}rrshare_{}-{}-.log'.format(
        log_path,
        os.sep,
        os.path.basename(sys.argv[0]).split('.py')[0],
        str(datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S'))
    )
except:
    _name = '{}{}rrshare-{}-.log'.format(
        log_path,
        os.sep,
        str(datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S'))
    )

logging.basicConfig(
    level=logging.WARNING,
    format='%(asctime)s RRSHARE>>> %(message)s',
    datefmt='%H:%M:%S',
    filename=_name,
    filemode='w',
)
console = logging.StreamHandler()
console.setLevel(logging.WARNING)
formatter = logging.Formatter('RRSHARE>> %(message)s')
console.setFormatter(formatter)
logging.getLogger('').addHandler(console)

logging.info('start RRSHARE')


def rq_util_log_debug(logs, ui_log=None, ui_progress=None):
    logging.debug(logs)


def rq_util_log_info(logs, ui_log=None, ui_progress=None, ui_progress_int_value=None):
    logging.warning(logs)

    # 给GUI使用，更新当前任务到日志和进度
    if ui_log is not None:
        if isinstance(logs, str):
            ui_log.emit(logs)
        if isinstance(logs, list):
            for iStr in logs:
                ui_log.emit(iStr)

    if ui_progress is not None and ui_progress_int_value is not None:
        ui_progress.emit(ui_progress_int_value)


def rq_util_log_expection(logs, ui_log=None, ui_progress=None):
    logging.exception(logs)
    

if __name__ == "__main__":
    rq_util_log_info('test loging info ok!')
    rq_util_log_debug('debug ?')
    



# -*- coding:utf-8 -*-
"""
Date: 2022/1/26 13:10
Desc: 申万指数-申万一级、二级和三级
http://www.swsindex.com/IdxMain.aspx
https://legulegu.com/stockdata/index-composition?industryCode=851921.SI
"""
import time
import json
import pandas as pd
import requests
from bs4 import BeautifulSoup


def sw_index_daily(
    symbol: str = "801011",
    start_date: str = "2019-12-01",
    end_date: str = "2020-12-07",
) -> pd.DataFrame:
    """
    申万指数一级和二级日频率行情数据
    http://www.swsindex.com/idx0200.aspx?columnid=8838&type=Day
    :param symbol: 申万指数
    :type symbol: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 申万指数日频率行情数据
    :rtype: pandas.DataFrame
    """
    #start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    #end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "http://www.swsindex.com/excel2.aspx"
    params = {
        "ctable": "swindexhistory",
        "where": f" swindexcode in ('{symbol}') and BargainDate >= '{start_date}' and BargainDate <= '{end_date}'",
    }
    r = requests.get(url, params=params)
    soup = BeautifulSoup(r.text, "html5lib")
    data = []
    table = soup.findAll("table")[0]
    rows = table.findAll("tr")
    for row in rows:
        cols = row.findAll("td")
        if len(cols) >= 10:
            symbol = cols[0].text
            index_name = cols[1].text
            date = cols[2].text
            open_ = cols[3].text
            high = cols[4].text
            low = cols[5].text
            close = cols[6].text
            vol = cols[7].text
            amount = cols[8].text
            change_pct = cols[9].text
            data.append(
                {
                    "index_code": symbol.replace(",", ""),
                    "index_name": index_name.replace(",", ""),
                    "date": date.replace(",", ""),
                    "open": open_.replace(",", ""),
                    "high": high.replace(",", ""),
                    "low": low.replace(",", ""),
                    "close": close.replace(",", ""),
                    "vol": vol.replace(",", ""),
                    "amount": amount.replace(",", ""),
                    "change_pct": change_pct.replace(",", ""),
                }
            )
    temp_df = pd.DataFrame(data)
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    temp_df["open"] = pd.to_numeric(temp_df["open"])
    temp_df["high"] = pd.to_numeric(temp_df["high"])
    temp_df["low"] = pd.to_numeric(temp_df["low"])
    temp_df["close"] = pd.to_numeric(temp_df["close"])
    temp_df["vol"] = pd.to_numeric(temp_df["vol"])
    temp_df["amount"] = pd.to_numeric(temp_df["amount"])
    temp_df["change_pct"] = pd.to_numeric(temp_df["change_pct"])
    return temp_df


def sw_index_daily_indicator(
    symbol: str = "801011",
    start_date: str = "2019-12-01",
    end_date: str = "2021-09-07",
    data_type: str = "Day",
) -> pd.DataFrame:
    """
    申万一级和二级行业历史行情指标
    http://www.swsindex.com/idx0200.aspx?columnid=8838&type=Day
    :param symbol: 申万指数
    :type symbol: str
    :param start_date: 开始时间
    :type start_date: str
    :param end_date: 结束时间
    :type end_date: str
    :param data_type: choice of {"Day": 日报表, "Week": 周报表}
    :type data_type: str
    :return: 申万指数不同频率数据
    :rtype: pandas.DataFrame
    """
    #start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    #end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "http://www.swsindex.com/excel.aspx"
    params = {
        "ctable": "V_Report",
        "where": f" swindexcode in ('{symbol}') and BargainDate >= '{start_date}' and BargainDate <= '{end_date}' and type='{data_type}'",
    }
    r = requests.get(url, params=params)
    soup = BeautifulSoup(r.text, "html5lib")
    data = []
    table = soup.findAll("table")[0]
    rows = table.findAll("tr")
    for row in rows:
        cols = row.findAll("td")
        if len(cols) >= 14:
            symbol = cols[0].text
            index_name = cols[1].text
            date = cols[2].text
            close = cols[3].text
            volume = cols[4].text
            chg_pct = cols[5].text
            turn_rate = cols[6].text
            pe = cols[7].text
            pb = cols[8].text
            v_wap = cols[9].text
            turnover_pct = cols[10].text
            float_mv = cols[11].text
            avg_float_mv = cols[12].text
            dividend_yield_ratio = cols[13].text
            data.append(
                {
                    "index_code": symbol,
                    "index_name": index_name,
                    "date": date,
                    "close": close,
                    "volume": volume,
                    "chg_pct": chg_pct,
                    "turn_rate": turn_rate,
                    "pe": pe,
                    "pb": pb,
                    "vwap": v_wap,
                    "float_mv": float_mv,
                    "avg_float_mv": avg_float_mv,
                    "dividend_yield_ratio": dividend_yield_ratio,
                    "turnover_pct": turnover_pct,
                }
            )
    temp_df = pd.DataFrame(data)
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    temp_df["close"] = pd.to_numeric(temp_df["close"])
    temp_df["volume"] = temp_df["volume"].apply(lambda x: x.replace(",", ""))
    temp_df["volume"] = pd.to_numeric(temp_df["volume"])
    temp_df["chg_pct"] = pd.to_numeric(temp_df["chg_pct"])
    temp_df["turn_rate"] = pd.to_numeric(temp_df["turn_rate"])
    temp_df["pe"] = pd.to_numeric(temp_df["pe"])
    temp_df["pb"] = pd.to_numeric(temp_df["pb"])
    temp_df["vwap"] = pd.to_numeric(temp_df["vwap"])
    temp_df["float_mv"] = temp_df["float_mv"].apply(lambda x: x.replace(",", ""))
    temp_df["float_mv"] = pd.to_numeric(
        temp_df["float_mv"],
    )
    temp_df["avg_float_mv"] = temp_df["avg_float_mv"].apply(
        lambda x: x.replace(",", "")
    )
    temp_df["avg_float_mv"] = pd.to_numeric(temp_df["avg_float_mv"])
    temp_df["dividend_yield_ratio"] = pd.to_numeric(temp_df["dividend_yield_ratio"])
    temp_df["turnover_pct"] = pd.to_numeric(temp_df["turnover_pct"])
    return temp_df



if  __name__ ==  "__main__":
    daily = sw_index_daily(symbol="801995", start_date="2022-04-01", end_date="2022-05-11")
    print(daily)
    indicator = sw_index_daily_indicator(symbol="801010", start_date="2022-04-01", end_date="2022-06-09")
    print(indicator)	

    


# coding:utf-8
import asyncio
from http import client
import asyncpg
import psycopg2
from rrshare.rqUtil import setting


host = setting['IP_DATABASE_ALIYUN']
db_name = setting["DATABASE_RRSHARE"]
passwd = setting["PGSQL_PASSWORD"]
uri = f'postgresql://postgres:{passwd}@{host}:5432/{db_name}'

# connect client
def rq_util_sql_postgres_setting(uri=uri):
    # 使用uri代替ip,port的连接方式
    # 这样可以对postgresql进行加密:
    # uri=postgresql://user:passwor@ip:port/daatbase
    #with  psycopg2.connect(uri) as conn:
    client = psycopg2.connect(uri)
    #print(client)
    return client

def show_sql_conn_count():
    """ select state, count(*) from pg_stat_activity  where pid <> pg_backend_pid() group by 1 order by 1;"""
    sql = str("select count(*) from pg_stat_activity;")
    with  rq_util_sql_postgres_setting() as conn:
        print(conn)
        cur = conn.cursor()
        cur.execute(sql)
        res = cur.fetchall()
        print(res)
        
    
    

if __name__ == '__main__':
    # test client
    print(rq_util_sql_postgres_setting())
    show_sql_conn_count()



#coding: utf-8
'''
功能：计算swl（L1，L2,L3）行业指数的不同时段（periods =[5,20,60,120,250]）的回报率rt和PRS
依赖：swl的各行业指数的日行情，tusharepro，->swl_day, swl_valuation
数据库： postgresql12
tips: df.round(2)  ;  sql-> "".jion(list), rq_util_get_pre_trade_date ; rq_util_get_last_tradedate
logging replace print
swl_day, list from rrshare , swl_rs_valuation save to rrfactor
'''
import time
import pandas as pd

import warnings
warnings.filterwarnings("ignore")
import logging
logging.basicConfig(level=logging.INFO, format=' %(asctime)s- %(levelname)s-%(message)s')

from rrshare.rqUtil import (rq_util_log_debug, rq_util_log_expection, rq_util_log_info)
from rrshare.rqUtil import (rq_util_date_today, trade_date_sse,rq_util_get_trade_range,
                            rq_util_get_last_tradedate, rq_util_get_pre_trade_date)
from rrshare.rqUtil import (PERIODS, is_trade_time_secs_cn)
from rrshare.rqUtil.rqPgsql import (PgsqlClass, client_pgsql, read_data_from_pg, read_table_from_pg,read_sql_from_pg, save_data_to_postgresql)
from rrshare.rqFetch.rqCodeName import (stock_code_to_name, stock_code_belong_swl_name, swl_index_to_name)
from rrshare.rqFetch import Swsindex
from rrshare.rqUtil import (PERIODS , SWL_LEVEL)


L = SWL_LEVEL().LEVEL
periods = PERIODS().PERIODS
print(L, periods)
NList = PERIODS().PERIODS
N1List = list(map(lambda x: x-1,NList))
client_rrshare = client_pgsql('rrshare')
client_rrfactor = client_pgsql('rrfactor')


class SwlPRS(object): #TODO
    
    def __init__(self, level):
        self.level = level

    def swl_PRS_level(self, period):
        #cols = "trade_date, index_symbol, name_level, level, change_pct" #, pe, pb"
        start_date = rq_util_get_pre_trade_date(rq_util_get_last_tradedate(), period)
        sql = f"SELECT  * FROM swl_day WHERE trade_date >= '{start_date}' ;"
        df = pd.read_sql_query(sql=sql, con=client_rrshare)
        print(df)
        
       
        df = df[df['level']==self.level]
        df = df[['index_symbol','name_level', 'change_pct']]
                
        rt = df.groupby(by='index_symbol').sum()
        rt['rank'] = rt['change_pct'].rank(ascending=False)
        rt['rs'] = round(100*(1 - rt['rank']/len(rt)), 2)
        rt = rt.rename(columns={'change_pct': f'rt_{str(period)}'})
        rt = rt.rename(columns={'rs': f'rs_{str(period)}','rank':f'rank_{str(period)}'})
        rt =rt.round(2)
        #print(rt)
        df_name = swl_index_to_name(swl_level=self.level).set_index('index', drop=False)  # keep index in columns
        #df_name['name'] = df_name['name_level']
        #print(df_name)
        RS = pd.concat([df_name, rt], axis=1, sort=True)
        RS = RS[['index_symbol','name_level', f'rt_{str(period)}',f'rs_{str(period)}']]
        print(RS)
        return RS
    

    def swl_Prs_all_periods(self):
        RS = pd.DataFrame()
        for period in periods:
            rs = self.swl_PRS_level(period)
            RS = pd.concat([RS, rs], axis=1, sort=True)
        #print(RS)
        save_data_to_postgresql(RS,f'swl_rs_{self.level}',client_rrfactor)
        logging.info(f'写入数据库的表swl_rs_{self.level}, ok')


    def swl_rt_all_periods(self):
        RT = pd.DataFrame()
        level2 = L[1]
        for period in periods:
            p = period -1
            cols = "trade_date, index, name_level, level, pct_change, pe, pb"
            start_date = rq_util_get_pre_trade_date(rq_util_get_last_tradedate(), p)
            df = read_sql_from_pg(start_date=start_date, data=cols, table_name='swl_day',client=client_rrshare)
            df = df[df['level']==level2]
            df = df[['index','name_level', 'pct_change']]
            rt = df.groupby(by='index').sum()
            rt = rt.rename(columns={'pct_change': f'rt_{str(p)}'})
            rt =rt.round(2)
            #print(rt)
            RT = pd.concat([RT, rt], axis=1, sort=True)
        
        df_name = swl_index_to_name(swl_level=level1).set_index('index', drop=False)  # keep index in columns
        #df_name['name'] = df_name['name_level']
        print(df_name)
        RT = pd.concat([df_name, RT], axis=1, sort=True)
        logging.info(f'\n {RT}')
        #print(RT)
        save_data_to_postgresql(RT,f'swl_rt_{level1}',client_rrfactor)
        logging.info(f'写入数据库的表swl_rt_{level1}, ok')
        
        return RT


    def swl_valuation(self):
        df = read_table_from_pg(period=1,table_name='swl_day', client=client_rrshare)
        df = df[['trade_date', 'index', 'name_level','level','close','pct_change', 'pe', 'pb']]
        df = df[df['level']==self.level]
        df = df.set_index('index', drop=False)  # keep index in columns
        df = df.sort_values(by='pct_change', ascending=False)
        logging.info(f'\n {df}')
        return df


    def save_swl_valuation_pg(self):
        df =self.swl_valuation()
        save_data_to_postgresql(df, f'swl_valuation_{self.level}', client_rrfactor)
        logging.info(f'写入数据库的表swl_valuation_{self.level}, ok')


    def swl_rs_valuation(self):
        RS = read_data_from_pg(table_name=f'swl_rs_{self.level}',client=client_rrfactor)
        rs_cols = [f'rs_{x}' for x in periods]
        cols = ['index','name_level'] + rs_cols 
        RS = RS[cols]
        logging.info(RS)
        valuation = read_data_from_pg(f'swl_valuation_{self.level}', client_rrfactor)
        df = pd.merge(RS, valuation , on='index')
        logging.info(f'\n {df}')
        return df


    def save_swl_rs_valuation_pg(self):
        df =self.swl_rs_valuation()
        save_data_to_postgresql(df, f'swl_rs_valuation_{self.level}', client_rrfactor)
        logging.info(f'写入数据库的表swl_rs_valuation_{self.level}, ok')


def swl1_index_realtime_prs():
    level1=L[0]
    rt = read_data_from_pg(f'swl_rt_L1', client_rrfactor)
    #print(rt)
    new_rt  = Swsindex().get_swsindex_L1_realtime()[['index','pct_change','trade_date']] 
    print(new_rt)
    cols_1 = [ 'rt_'+ str(int(x -1)) for x in periods] 
    cols = ['rt_' + str(x) for x in periods]
    col_dict = dict(zip(cols,cols_1))
    #print(cols_1,cols)
    df = pd.merge(rt, new_rt, on='index', how='right' )
    for k, v in col_dict.items():
        df[k] = df[v] + df['pct_change']
    
    col_rs = ['rs_5','rs_10','rs_20','rs_60','rs_120', 'rs_250']
    col_rs_dict = dict(zip(col_rs, cols))
    #print(col_rs_dict)
    for k, v in col_rs_dict.items():
        df[k] = 100*df[v].rank(axis=0,ascending=True, pct=True)

    cols_prs = ['index','name_level','trade_date', 'pct_change',\
                'rs_5','rs_10','rs_20','rs_60','rs_120', 'rs_250',\
               ]
    df = df[cols_prs].round(2)
    df.sort_values(by='pct_change', ascending=False, inplace=True)
    save_data_to_postgresql(df, f'swl1_rs_{level1}', client_rrfactor)
    logging.info(f"\n {df}")
    return df


def update_swl_rs_valuation(L=L[:2]):
    for l in L:
        prs = SwlPRS(l)
        prs.swl_Prs_all_periods()
        prs.save_swl_valuation_pg()
        prs.save_swl_rs_valuation_pg()


def update_swl_all_main():
    prs = SwlPRS("L2")
    prs.swl_rt_all_periods()

    prs.swl_PRS_level(period=20)
    prs.swl_Prs_all_periods()
    prs.swl_valuation()
    update_swl_rs_valuation(L)


if __name__ == '__main__':
    
    prs = SwlPRS("L2")
    prs.swl_PRS_level(period=20)
    #update_swl_all_main()
    pass



#coding: utf-8
#save stock_day, swl_day
from rrshare.rqSU.save_tusharepro_pg import (
                                            #rq_save_stock_day_pg,
                                             rq_save_swl_day_pg,
                                             rq_save_stock_list_pg,
                                             rq_save_stock_day_adj_fillna_pg
                                             )


#from rrshare.rqSU.save_industry_swl_list import rq_save_swl_industry_list_stock_belong

#from rrshare.rqSU.save_swsindex_realtime_pg import save_swl1_realtime





# coding : utf-8
import time
import datetime
from rrshare.rqUtil.rqParameter import TRADE_TIMES
from rrshare.rqUtil.rqDate_trade import trade_date_sse
from rrshare.rqUtil.rqDate import rq_util_date_today


def is_trade_time_secs_cn(time_: str=time.ctime()) -> bool:
    #print(time_)
    #print(datetime.datetime.now().strftime('%H:%M:%S'))
    time_hour_min =  time_.split(" ")[3]
    #print(time_hour_min)
    #print(TRADE_TIMES().TRADE_TIME_AM[0])
    con1 = (time_hour_min >= TRADE_TIMES().TRADE_TIME_AM_pre[0]) and (time_hour_min <= TRADE_TIMES().TRADE_TIME_AM_pre[1])
    #print(con1)
    con2 = (time_hour_min >= TRADE_TIMES().TRADE_TIME_PM_pre[0]) and (time_hour_min <= TRADE_TIMES().TRADE_TIME_PM_pre[1])
    #print(con2)
    con = con1 or con2
    return  con


def is_before_tradetime_secs_cn(time_: str=time.ctime()) -> bool:
    #print(time_)
    time_hour_min =  time_.split(" ")[3]
    #print(time_hour_min)
    return (time_hour_min < TRADE_TIMES().TRADE_TIME_AM_pre[0])


def is_tradeday_and_market_opened(time_: str=time.ctime()) -> bool:
    #print(time_)
    time_hour_min =  time_.split(" ")[3]
    #print(time_hour_min)
    today_ = rq_util_date_today().strftime('%Y-%m-%d')
    is_trade_day = today_ in trade_date_sse
    #print(is_trade_day)
    market_opened =  time_hour_min > TRADE_TIMES().TRADE_TIME_AM[0]
    print(market_opened)
    return is_trade_day and market_opened


if __name__ ==  "__main__":
    #print(TRADE_TIMES().TRADE_TIME_AM_pre[0])
    #print(is_trade_time_secs_cn())
    #print(is_before_tradetime_secs_cn())
    print(is_tradeday_and_market_opened())




    



# coding: utf-8
import configparser
import json
import os
from multiprocessing import Lock
from rrshare.rqUtil.rqLocalize import rq_path, setting_path
from rrshare.rqUtil.config_setting import setting
from rrshare.rqUtil.rqSql import rq_util_sql_postgres_setting

# rrsdk or rrdata有一个配置目录存放在 ~/.rrsdk
# 如果配置目录不存在就创建，主要配置都保存在config.json里面
# 文件的创建步骤，他还会创建一个setting的dir

POSTGRESQL_URI = setting["POSTGRESQL"]
print(POSTGRESQL_URI)
CONFIGFILE_PATH = '{}{}{}'.format(setting_path, os.sep, 'config.ini')
#print(CONFIGFILE_PATH)

class rq_Setting():

    def __init__(self, uri=None):
        self.lock = Lock()
        self.postgres_uri = uri or self.get_postgres()
        self.username = None
        self.password = None


    def get_postgres(self):
        config = configparser.ConfigParser()
        if os.path.exists(CONFIGFILE_PATH):
            config.read(CONFIGFILE_PATH)
            try:
                res = config.get('POSTGRESQL', 'uri')
            except:
                res =POSTGRESQL_URI
        else:
            config = configparser.ConfigParser()
            config.add_section('POSTGRESQL')
            config.set('POSTGRESQL', 'uri',POSTGRESQL_URI)
            f = open('{}{}{}'.format(setting_path, os.sep, 'config.ini'), 'w')
            config.write(f)
            res =POSTGRESQL_URI
        return res


    def get_config(self,
            section='POSTGRESQL',
            option='uri',
            default_value=POSTGRESQL_URI):
        try:
            config = configparser.ConfigParser()
            config.read(CONFIGFILE_PATH)
            return config.get(section, option)
        except:
            res = self.client.rrquant.usersetting.find_one(
                {'section': section})
            if res:
                return res.get(option, default_value)
            else:
                self.set_config(section, option, default_value)
                return default_value


    def set_config(
            self,
            section='POSTGRESQL',
            option='uri',
            default_value=POSTGRESQL_URI
    ):
        """[summary]
        Keyword Arguments:
            section {str} -- [description] (default: {'POSTGRESQL'})
            option {str} -- [description] (default: {'uri'})
            default_value {[type]} -- [description] (default: {DEFAULT_DB_URI})
        Returns:
            [type] -- [description]
        """
        t = {'section': section, option: default_value}
        self.client.rrquant.usersetting.update(
            {'section': section}, {'$set': t}, upsert=True)

        # if os.path.exists(CONFIGFILE_PATH):
        #     config.read(CONFIGFILE_PATH)
        #     self.lock.release()
        #     return self.get_or_set_section(
        #         config,
        #         section,
        #         option,
        #         default_value,
        #         'set'
        #     )

        #     # 排除某些IP
        #     # self.get_or_set_section(config, 'IPLIST', 'exclude', [{'ip': '1.1.1.1', 'port': 7709}])

        # else:
        #     f = open(CONFIGFILE_PATH, 'w')
        #     config.add_section(section)
        #     config.set(section, option, default_value)

        #     config.write(f)
        #     f.close()
        #     self.lock.release()
        #     return default_value

    def get_or_set_section(self,
            config,
            section,
            option,
            DEFAULT_VALUE,
            method='get'):
        try:
            if isinstance(DEFAULT_VALUE, str):
                val = DEFAULT_VALUE
            else:
                val = json.dumps(DEFAULT_VALUE)
            if method == 'get':
                return self.get_config(section, option)
            else:
                self.set_config(section, option, val)
                return val
        except:
            self.set_config(section, option, val)
            return val


    def env_config(self):
        return os.environ.get("POSTGRESURI", None)

    @property
    def client(self):
        return rq_util_sql_postgres_setting(self.postgres_uri)


rqSETTING = rq_Setting()
DATABASE = rqSETTING.client
#print(DATABASE)



import time
import datetime
import os
import pickle
import pandas as pd
import numpy as np
from decimal import Decimal
import warnings
warnings.filterwarnings("ignore")
import logging
logging.basicConfig(level=logging.INFO, format=' %(asctime)s- %(levelname)s-%(message)s')

from rrshare.rqUtil import (rq_util_log_debug, rq_util_log_expection,
                                                     rq_util_log_info)
from rrshare.rqUtil import (rq_util_date_today, rq_util_get_trade_range, rq_util_get_last_tradedate, rq_util_get_pre_trade_date)
from rrshare.rqUtil import (PERIODS, is_trade_time_secs_cn, is_tradeday_and_market_opened)
from rrshare.rqUtil import (client_pgsql, read_data_from_pg, read_sql_from_pg)
from rrshare.rqFetch import Swsindex
from rrshare.rqUtil.rqParameter import START_DATE_COUNT

N = START_DATE_COUNT
NList = PERIODS().PERIODS
N1List = list(map(lambda x: x-1,NList))
client_rrshare = client_pgsql('rrshare')
client_rrfactor = client_pgsql('rrfactor')


USERPATH = os.path.expanduser('~')
#print(USERPATH)
file_path = f'{USERPATH}/.rrsdk/data'
#file_path_name = f"{file_path}/{table_name}_{N}.pkl"

def swl_RT_HH_MA_pre(table_name='swl_day', N=N, level="L2"):
    """caculate i-1 swl_day rt_ma
       prepare to next day  day rs_OH_ma
       swl_day has last some data N = 250 + 10
       迭代， 只算一次， 速度快。
    """
    trade_date = rq_util_get_last_tradedate()
    start_date = rq_util_get_pre_trade_date(trade_date,N)
    #print(start_date, trade_date, level)
    '''
    sql = f"SELECT * FROM {table_name} WHERE level='{level}' AND trade_date >= '{start_date}'"
    print(sql)
    df = pd.read_sql_query(sql,con=client_rrshare)
    print(df)
    df.to_pickle(f"~/.rrsdk/data/sw_{level}_day_250.pkl")
    '''
    file_path_name = f"{file_path}/{table_name}_{N}.pkl"
    df = pd.read_pickle(file_path_name)
    #print(df)
    #print(df.columns)
    df = df[['trade_date','name_level', 'open', 'high', 'low', 'close',
       'change_pct', 'vol', 'amount', 'index_code','level' ]]
    cols = ['open', 'high', 'low', 'close','change_pct', 'vol', 'amount']
    for i in cols:
        df[i] = pd.to_numeric(df[i], errors='coerce')
    df_i = df.set_index(['trade_date','index_code'])
    df_i.drop_duplicates(keep='last',inplace=True)
    df_un = df_i.unstack()
    vol_chg = 100*(df_un['vol'] / df_un['vol'].rolling(49).mean() - 1).unstack()
    df_fill = df_un.stack().reset_index()
    #print(df_fill)
    ma=dict()
    rt=dict()
    for i in N1List:
        ma[i] = (df_un['close'].rolling(i).mean()).unstack()
        rt[i] = df_un['change_pct'].rolling(i).sum().unstack()
    H = (df_un['high']).expanding().max()
    HH = H.unstack()
    L = df_un['low'].expanding().min()
    LL = L.unstack()
    close = df_un['close'].unstack()
    pct_chg = df_un['change_pct'].unstack()
    name_level = df_un['name_level'].unstack()
    pct_ma_rs = pd.DataFrame({
    'close_pre':close,'name_level':name_level, 'pct_chg_pre': pct_chg, 
    'ma4':ma[4],'ma9':ma[9],'ma19':ma[19], 'ma59':ma[59],'ma119':ma[119],'ma249':ma[249],
    'rt4':rt[4],'rt9':rt[9],'rt19':rt[19],'rt59':rt[59],'rt119':rt[119], 'rt249': rt[249], 
    'vol_chg_pre': vol_chg,'H':HH,'L':LL, 
    })
    data = pct_ma_rs.reset_index()
    df= data[data['trade_date'] == trade_date]
    table_name_L = f"swl_{level}_RT_HH_MA_pre"
    print(table_name_L)
    df.to_sql(table_name_L,con=client_rrfactor, if_exists='replace')
    return df


def swl_RS_OH_MA_last(table_name='swl_day', N=N, level="L2"):
    """caculate i swl_day rt_ma
       prepare to next day  day rs_OH_ma
       swl_day has last some data N = 250 + 10
       pd.to_numeric()  !!!
       迭代， 只算一次， 速度快。
    """
    trade_date = rq_util_get_last_tradedate()
    start_date = rq_util_get_pre_trade_date(trade_date,N)
    print(start_date, trade_date)
    '''
    sql = f"SELECT * FROM {table_name} WHERE level='{level}' AND trade_date >= '{start_date}'"
    print(sql)
    df = pd.read_sql_query(sql,con=client_rrshare)
    print(df)
    #df.to_pickle(f"~/.rrsdk/data/sw_{level}_day_250.pkl")
    '''
    file_path_name = f"{file_path}/{table_name}_{N}.pkl"
    df = pd.read_pickle(file_path_name)
    #print(df)
    #print(df.columns)
    df = df[['trade_date','name_level', 'open', 'high', 'low', 'close',
       'change_pct', 'vol', 'amount', 'index_code','level' ]]
    cols = ['open', 'high', 'low', 'close','change_pct', 'vol', 'amount']
    for i in cols:
        df[i] = pd.to_numeric(df[i], errors='coerce')
    df_i = df.set_index(['trade_date','index_code'])
    df_i.drop_duplicates(keep='last',inplace=True)
    df_un = df_i.unstack()
    vol_chg = 100*(df_un['vol'] / df_un['vol'].rolling(50).mean() - 1).unstack()
    df_fill = df_un.stack().reset_index()
    #print(df_fill)
    ma=dict()
    rt=dict()
    rs=dict()
    for i in NList:
        ma[i] = (df_un['close'].rolling(i).mean()).unstack()
        rt[i] = df_un['change_pct'].rolling(i).sum().unstack()
        rs[i]= 100*rt[i].unstack(0).rank(axis=1,ascending=True, pct=True).unstack()
    H = (df_un['high']).expanding().max()
    HH = H.unstack()
    L = df_un['low'].expanding().min()
    LL = L.unstack()
    OH = 100*(df_un['close']/H).unstack()
    OL = 100*(df_un['close']/L - 1).unstack()
    close = df_un['close'].unstack()
    pct_chg = df_un['change_pct'].unstack()
   
    name_level = df_un['name_level'].unstack()
    pct_ma_rs = pd.DataFrame({
    'close':close,'name_level':name_level, 'pct_chg': pct_chg, 
    'ma5':ma[5],'ma10':ma[10],'ma20':ma[20], 'ma60':ma[60],'ma120':ma[120],'ma250':ma[250],
    'rt5':rt[5],'rt10':rt[10],'rt20':rt[20],'rt60':rt[60],'rt120':rt[120], 'rt250': rt[250],
    'rs5':rs[5],'rs10':rs[10],'rs20':rs[20],'rs60':rt[60],'rs120':rs[120], 'rs250': rt[250],
    'vol_chg':vol_chg,'OH':OH,'OL':OL,'H':HH,'L':LL, 
    })
    data = pct_ma_rs.reset_index()
    #print(data)
    df = data[data['trade_date'] == trade_date]
    #print(df)
    table_name_L=f"swl_{level}_RS_OH_MA_last"
    df.to_sql(table_name_L,con=client_rrfactor, if_exists='replace')
    return df


def swl_RS_OH_MA(level="L2"):
    df = read_data_from_pg(table_name=f'swl_{level}_RS_OH_MA_last',client=client_rrfactor)
    logging.info(f'\n {df.head()}')
    df.rename(columns={'close_pre':'close','adj_factor_pre':'adj_factor','pct_chg_pre':'pct_chg'}, inplace=True)
    df['OH'] = 100*(df['close']/df['H'])
    #df['OH'] = df['OH'].apply(lambda x: Decimal(x).quantize(Decimal((0.00))))
    df['OL'] = 100*(df['close']/df['L'] - 1)
    #df['OL'] = df['OL'].apply(lambda x: Decimal(x).quantize(Decimal((0.00))))
    col_rt =['rt5', 'rt10', 'rt20', 'rt60','rt120', 'rt250']
    col_rs = ['rs_5','rs_10','rs_20','rs_60','rs_120', 'rs_250']
    col_rs_dict = dict(zip(col_rs,col_rt))
    for k, v in col_rs_dict.items():
        df[k] = 100*df[v].rank(axis=0,ascending=True, pct=True)
        cols = ['trade_date', 'code','close', 'pct_chg',\
                'ma5','ma10','ma20', 'ma60', 'ma120','ma250',\
                'rs_5','rs_10','rs_20','rs_60','rs_120', 'rs_250',\
                'rt5', 'rt10', 'rt20', 'rt60','rt120', 'rt250',\
                'H','L','OH','OL']
        #print(cols)
        #df = df[cols].round(2)
        print(df)
        print(df.columns)
        table_name_L = f"swl_{level}_RS_OH_MA"
        print(table_name_L)
        df.to_sql(table_name_L, con=client_pgsql('rrfactor'), if_exists='replace')
        logging.info(f'\n {df.head()}')
        return df


def swl_RS_OH_MA_new(level="L2"):
    """ fast and right pkl or h5 or sql
    """
    #df = pd.read_pickle('/home/rome/.rrshare/data/stock_RT_HH_MA_pre.pkl')
    df = pd.read_sql_table(f"swl_{level}_RT_HH_MA_pre", client_rrfactor)
    logging.info(df.head())
    df_last_date =str( max(set(list(df['trade_date'].values))))[0:10]
    #print(df_last_date)
    today_ = rq_util_date_today().strftime('%Y-%m-%d') 
    trade_date = today_  if is_tradeday_and_market_opened() else rq_util_get_last_tradedate()
    #print(trade_date)

    if (trade_date > df_last_date): 
        #fetch_realtime_price from rqFetch
        df_p = Swsindex().fetch_swsindex_L1_L2_realtime(level)
        print(df_p)
        df_p = df_p.drop(columns =['trade_date','amount','open'], axis=1)
        
        df = pd.merge(df,df_p, how='left', on='index_code')
        #print(df)
        #vol_chg = 100*((df['vol']*df['adj_factor']) / (df['vol']*df['adj_factor']).rolling(50).mean() - 1)
        col_ma_pre = ['ma4','ma9', 'ma19', 'ma59', 'ma119', 'ma249']
        col_ma = ['ma5','ma10','ma20', 'ma60','ma120','ma250']
        col_rt_pre =['rt4', 'rt9', 'rt19', 'rt59','rt119', 'rt249']
        col_rt =['rt5', 'rt10', 'rt20', 'rt60','rt120', 'rt250']
        col_ma_dict = dict(zip(col_ma,col_ma_pre))
        #print(col_ma_dict)
        col_rt_dict = dict(zip(col_rt, col_rt_pre))
        for  k, v in col_ma_dict.items():
            n = int(k[2:])
            #print(n)
            df[k] = ((n-1)*df[v] + df['close']) / n
            #print(df[k])
            df[k] = df[k].apply(lambda x: Decimal(x).quantize(Decimal(('0.00'))))
        for  k, v in col_rt_dict.items():
            n = int(k[2:])
            df[k] = df[v] + df['change_pct']
        df.dropna(subset=['H','L','name'],inplace=True)   # TODO ???
        #print(df)
        vol_chg = 100*(df['vol']/(df['vol']).rolling(50).mean() - 1)  # TODO
       
        df['H'] = df.apply(lambda x: max(x.H, x.high), axis=1) 
       
        df['L'] = df.apply(lambda x: min(x.L, x.low), axis=1)
        df['OH'] = 100*(df['close']/df['H'])
        #df['OH'] = df['OH'].apply(lambda x: Decimal(x).quantize(Decimal((0.00))))
        df['OL'] = 100*(df['close']/df['L'] - 1)
        #df['OL'] = df['OL'].apply(lambda x: Decimal(x).quantize(Decimal((0.00))))

        #rsi= 100*rt[i].unstack(0).rank(axis=1,ascending=True, pct=True).unstack()
        col_rs = ['rs_5','rs_10','rs_20','rs_60','rs_120', 'rs_250']
        col_rs_dict = dict(zip(col_rs,col_rt))
        for k, v in col_rs_dict.items():
            df[k] = 100*df[v].rank(axis=0,ascending=True, pct=True)

        cols = ['index_code', 'close',  'change_pct',\
                'ma5','ma10','ma20', 'ma60', 'ma120','ma250',\
                'rs_5','rs_10','rs_20','rs_60','rs_120', 'rs_250',\
                'rt5', 'rt10', 'rt20', 'rt60','rt120', 'rt250',\
                'H','L','OH','OL']

        #print(cols)
        df = df[cols].round(2)
        print(df)
        #df.rename(columns={'date':'trade_date'}, inplace=True)
        table_name_L = f"swl_{level}_RS_OH_MA_new"
        #print(table_name_L)
        df.to_sql(table_name_L, con=client_rrfactor, if_exists='replace')
        logging.info(f"\n {df.head()}")
        return df
    else:
        return read_data_from_pg(table_name=f'swl_{level}_RS_OH_MA',client=client_rrfactor)


def update_swl_PRS_day():
    for l in ["L1","L2"]:
        swl_RT_HH_MA_pre(level=l)
        swl_RS_OH_MA_last(level=l)
        swl_RS_OH_MA(level=l)


def update_swl_PRS_new():
    for l in ["L1","L2"]:
        swl_RS_OH_MA_new(level=l)
    

if  __name__  == "__main__":
    update_swl_PRS_day()
    update_swl_PRS_new()


# coding: utf-8
import pandas as pd
import numpy as np
import os
import time
import datetime
import pymongo
import tushare as ts
from sqlalchemy import create_engine
import psycopg2  
import warnings
warnings.filterwarnings("ignore")

from rrshare.rqUtil.rqLogs import (rq_util_log_debug, rq_util_log_expection,
                                     rq_util_log_info)
from rrshare.rqUtil.rqCode import (rq_util_code_tosrccode, rq_util_code_tostr)
from rrshare.rqUtil.rqDate_trade import rq_util_get_trade_range
from rrshare.rqUtil.rqDate import (rq_util_date_str2int,rq_util_date_int2str)

from rrshare.rqUtil import (PgsqlClass, rq_util_get_last_tradedate)
from rrshare.rqFetch import (pro, fetch_stock_day_adj_fillna_from_tusharepro)


from rrshare.rqFetch import  fetch_stock_list_tusharepro
from rrshare.rqUtil.rqParameter import startDate
from rrshare.rqFetch.fetch_swl_index_daily_and_realtime import Swsindex, get_sw_index_daily_L1_L2,get_sw_index_daily_valuation_L1_L2

lastTD = rq_util_get_last_tradedate()
print(lastTD)

def create_pgsql_table(tabel_name='', tsql='', db_name='rrshare'):
    try:
        PgsqlClass().create_table(tabel_name, tsql)
    except Exception as e:
        print(e)

def client_pgsql(database='rrshare'):
    try:
        return PgsqlClass().client_pg(db_name=database)
    except Exception as e:
        print(e)

def save_data_to_postgresql(name_biao,data,if_exists='replace',client=client_pgsql()):
        data.to_sql(name_biao,client,index=False,if_exists=if_exists)
    
def load_data_from_postgresql(mes='',client=client_pgsql()):
    res=pd.read_sql(mes,client)
    return res

def read_data_from_pg(table_name='', client=client_pgsql()):
    res = pd.read_sql_table(table_name, client)
    return res


def rq_fetch_stock_list_pg(name_biao='stock_list'):
    mes='select * from '+name_biao+";"    
    try:   
        t=time.time()        
        res=load_data_from_postgresql(mes=mes)
        t1=time.time()
        rq_util_log_info('load '+ name_biao+ ' success,take '+str(round(t1-t,2))+' S')              
    except Exception as e:
        print(e)
        res=None
    return res


def rq_save_stock_list_pg():
    stock_list_l= pro.stock_basic(exchange_id='', is_hs='',list_status='L' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    #stock_list_D= pro.stock_basic(exchange_id='', is_hs='',list_status='D' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    #stock_list_P= pro.stock_basic(exchange_id='', is_hs='',list_status='P' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')          
    #stock_list=pd.concat([stock_list_l,stock_list_D],axis=0)
    #stock_list=pd.concat([stock_list_l,stock_list_P],axis=0)
    stock_list = stock_list_l
    stock_list['code']=stock_list['symbol']
    #print(df)
    try:   
        t=time.time()    
        save_data_to_postgresql('stock_list',stock_list)
        t1=time.time()
        rq_util_log_info('save stock_list data success,take '+str(round(t1-t,2))+' S') 
    except Exception as e:        
        print(e) 


def rq_save_swl_day_pg(start_date=startDate, table_name='swl_day'): #from 2015-01-04
    #table_name='swl_day'
    t = time.localtime(time.time())
    if int(time.strftime('%H%M%S',t))<160000:   #晚上9点之后在更新当天数据，以免不及时
        t = time.localtime(time.time()-3600*24)
        tS = time.strftime("%Y-%m-%d", t)
    else:
        tS = time.strftime("%Y-%m-%d", t)
    end_date=tS
    print(end_date)
    try:
        mes=f'select distinct trade_date FROM {table_name};'
        #mes='SELECT DISTINCT trade_date FROM swl_day;'
        trade_data_pg = load_data_from_postgresql(mes).trade_date.tolist()
        #print(trade_data_pg[-3:])
        for i in range(len(trade_data_pg)):
            trade_data_pg[i]=trade_data_pg[i].strftime("%Y-%m-%d")
            #trade_data_pg = list(map(lambda x: x.strftime('%Y-%m-%d'), trade_data_pg))
            #print(len(trade_data_pg))
            #print(trade_data_pg[-1:])
    except: #第一次运行
        trade_data_pg=list()
        #print(trade_data_pg[-3:])
    if isinstance(start_date,int):
        start_date=rq_util_date_int2str(start_date)
    elif len(start_date)==8:
        start_date=start_date[0:4]+'-'+start_date[4:6]+'-'+start_date[6:8]
        #print(start_date)
    trade_date=rq_util_get_trade_range(start_date,end_date)
    #print(trade_date)
    trade_date2=list(set(trade_date).difference(set(trade_data_pg))) #差集 在trade_date 中 不在trade_data_pg
    trade_date2.sort()
    print(trade_date2)

    if len(trade_date2)==0:
        rq_util_log_info('swl day is up to date and does not need to be updated')
    for i in trade_date2:
        print(i)
        try:
            t=time.time()
            df = get_sw_index_daily_L1_L2(trade_date=i)
            save_data_to_postgresql(table_name, df, 'append')
            t1=time.time()
            tt = round((t1-t),4)
            time.sleep(1)
            rq_util_log_info(f'save {i} {table_name} success,take {tt}S')
        except Exception as e:
            print(e)


def rq_save_swl_day_valucation_pg(start_date=startDate, table_name='swl_day_valuation'): #from 2015-01-04
    t = time.localtime(time.time())
    if int(time.strftime('%H%M%S',t))<160000:   #晚上4点之后在更新当天数据，以免不及时
        t = time.localtime(time.time()-3600*24)
        tS = time.strftime("%Y-%m-%d", t)
    else:
        tS = time.strftime("%Y-%m-%d", t)
    end_date=tS
    print(end_date)
    try:
        mes=f'select distinct trade_date FROM {table_name};'
        #mes='SELECT DISTINCT trade_date FROM swl_day;'
        trade_data_pg = load_data_from_postgresql(mes).trade_date.tolist()
        #print(trade_data_pg[-3:])
        for i in range(len(trade_data_pg)):
            trade_data_pg[i]=trade_data_pg[i].strftime("%Y-%m-%d")
            #trade_data_pg = list(map(lambda x: x.strftime('%Y-%m-%d'), trade_data_pg))
            #print(len(trade_data_pg))
            #print(trade_data_pg[-1:])
    except: #第一次运行
        trade_data_pg=list()
        #print(trade_data_pg[-3:])
    if isinstance(start_date,int):
        start_date=rq_util_date_int2str(start_date)
    elif len(start_date)==8:
        start_date=start_date[0:4]+'-'+start_date[4:6]+'-'+start_date[6:8]
        #print(start_date)
    trade_date=rq_util_get_trade_range(start_date,end_date)
    #print(trade_date)
    trade_date2=list(set(trade_date).difference(set(trade_data_pg))) #差集 在trade_date 中 不在trade_data_pg
    trade_date2.sort()
    print(trade_date2)

    if len(trade_date2)==0:
        rq_util_log_info('swl day is up to date and does not need to be updated')
    for i in trade_date2:
        print(i)
        try:
            t=time.time()
            df = get_sw_index_daily_valuation_L1_L2(trade_date=i)
            save_data_to_postgresql(table_name, df, 'append')
            t1=time.time()
            tt = round((t1-t),4)
            time.sleep(1)
            rq_util_log_info(f'save {i} {table_name} success,take {tt}S')
        except Exception as e:
            print(e)


if __name__ == '__main__':          
    #rq_save_stock_list_pg()
    rq_save_swl_day_pg()
    rq_save_swl_day_valucation_pg() 




# coding=utf8
import json
import pandas as pd
import tushare as ts

from rrshare.rqUtil import rq_util_get_last_tradedate
from rrshare.rqFetch import pro

def fetch_delist_stock(trade_date=None):
    df_P = pro.stock_basic(exchange='',list_status='P',fields='ts_code,symbol,name,list_status,delist_date,list_date')
    df_D = pro.stock_basic(exchange='',list_status='D',fields='ts_code,symbol,name,list_status,delist_date,list_date')
    df_DP = pd.concat([df_P, df_D], axis=0)
    df_DP['code'] = df_DP['ts_code'].apply(lambda x: x[0:6])
    if not trade_date:
        trade_date = rq_util_get_last_tradedate().replace('-','')
    df_DD = df_DP[df_DP['delist_date'] <= trade_date]
    df_DD_code = list(df_DD['code'].values)
    return df_DD_code

def fetch_stock_list_tusharepro():    
    stock_list_l= pro.stock_basic(exchange_id='', is_hs='',list_status='L' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    stock_list_D= pro.stock_basic(exchange_id='', is_hs='',list_status='D' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    stock_list_P= pro.stock_basic(exchange_id='', is_hs='',list_status='P' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')          
    stock_list=pd.concat([stock_list_l,stock_list_D],axis=0)
    stock_list=pd.concat([stock_list,stock_list_P],axis=0)
    return stock_list

def fetch_get_stock_list(trade_date=None,ts_code=True):
    df_L= pro.stock_basic(exchange_id='', is_hs='',list_status='L') # , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    #stock_list_P= pro.stock_basic(exchange_id='', is_hs='',list_status='P' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')          
    #stock_list=pd.concat([stock_list_l,stock_list_P],axis=0)
    if ts_code:
        return list(df_L['ts_code'].values)
    df_L['code'] = df_L['ts_code'].apply(lambda x: x[0:6])
    #print(df_L)
    return list(df_L['code'].values)

def fetch_get_stock_list_adj(trade_date=None, ts_code=False):
    if not trade_date:
        trade_date = rq_util_get_last_tradedate().replace('-','')
    adj = pro.adj_factor(trade_date=trade_date)
    if ts_code:
        return list(adj['ts_code'].values)
    adj['code'] = adj['ts_code'].apply(lambda x: x[0:6])
    return list(adj['code'].values)


def fetch_pro_bar(ts_code=None,asset="E", adj='hfq', start_date='20180101', end_date='20181011'): #TODO
    df = ts.pro_bar(ts_code=ts_code, asset=asset,adj=adj, start_date=start_date, end_date=end_date) 
    return df


if __name__ == '__main__':
    
    #print(fetch_delist_stock())
    print(len(fetch_get_stock_list()), fetch_get_stock_list(ts_code=True)[0:6])
    print(len(fetch_get_stock_list_adj()), fetch_get_stock_list_adj(ts_code=True)[-5:])
    #print(fetch_pro_bar(ts_code='000001.SH',asset="I", adj='qfq', start_date='20210101', end_date='20210507'))
    pass



import os
import subprocess
import pandas as pd 

from rrshare.rqUtil.rqDecorator import timer
from rrshare.rqUtil.rqDate_trade import rq_util_get_last_tradedate, rq_util_get_pre_trade_date
from rrshare.rqUtil.rqParameter import START_DATE_COUNT
from rrshare.rqUtil import client_pgsql


def read_daily_counts_from_pickle(table_name='stock_day_adj_fillna',N=START_DATE_COUNT):
    """  read large stockday N>250 need 10mins, cron read data save to pickle for speed"""
    USERPATH = os.path.expanduser('~')
    #print(USERPATH)
    file_path = f'{USERPATH}/.rrsdk/data'
    file_path_name = f"{file_path}/{table_name}_{N}.pkl"
    
    trade_date = rq_util_get_last_tradedate()
    start_date = rq_util_get_pre_trade_date(trade_date,N)
    if os.path.exists(file_path_name):
        try:
            df = pd.read_pickle(file_path_name)
            df = df.sort_values(by="trade_date")
            print(f"Read daily data from <{file_path_name}> \n {df}")
            return df
        except Exception as e:
            print(e)
            return pd.DataFrame()
    else:
        print(f"File <{file_path_name}> is not exists !")
        return 0
            


def is_daily_pickle_new(table_name='swl_day', N=START_DATE_COUNT):
    """check picle daily data is or not new! """
    USERPATH = os.path.expanduser('~')
    #print(USERPATH)
    file_path = f'{USERPATH}/.rrsdk/data'
    file_path_name = f"{file_path}/{table_name}_{N}.pkl"
    trade_date = rq_util_get_last_tradedate()
    start_date = rq_util_get_pre_trade_date(trade_date,N)
    if os.path.exists(file_path_name):
        try:
            df = pd.read_pickle(file_path_name)
            df = df.sort_values(by="trade_date")
            print(f"Read daily data from <{file_path_name}> \n {df}")
        except Exception as e:
            print(e)
    else:
        print(f"File <{file_path_name}> is not exists !")
        df = None
    if df is not None:
        print(df)
        df_last = df.copy()
        df_last_one = set(pd.to_datetime(df_last.trade_date.values, format='%Y-%m-%d'))
        last_date = str(max(df_last_one)).split(" ")[0]
        #print(last_date)
        flag = last_date == rq_util_get_last_tradedate()
        print(f" Daily data in <{file_path_name}> is New .") if flag else print("daily data in <{file_path_name}> not New, need update!")
        return flag
    else:
        return False
    

@timer
def save_daily_count_to_pickle(table_name='stock_day_adj_fillna', N=START_DATE_COUNT):
    """  read large stockday N>250 need 10mins, cron read data save to pickle for speed"""
    USERPATH = os.path.expanduser('~')
    #print(USERPATH)
    file_path = f'{USERPATH}/.rrsdk/data'
    if not os.path.exists(file_path):
        subprocess.run(['mkdir','-p', file_path])
    file_path_name = f"{file_path}/{table_name}_{N}.pkl"
    trade_date = rq_util_get_last_tradedate()
    start_date = rq_util_get_pre_trade_date(trade_date,N)
    print(f"You Input Daily counts is {N} > 250, Good!") if N > 250 else print(f"Ckeck Daily counts : {N} < 250")
    if N < 250  and (table_name.split("_")[0] == "stock"):
        print(" Warninng: Are you Check stock Daily counts , Sure > = 250 !")
    if N < 260 and (table_name.split("_")[0] == "swl"):
        print(" Warninng: Are you Check swl Daily counts , Sure > = 260 !")
        
    
    if not is_daily_pickle_new(table_name=table_name, N=N):
        sql = f"SELECT * FROM {table_name} WHERE trade_date >= '{start_date}'"
        df = pd.read_sql_query(sql, client_pgsql("rrshare"))
        df.to_pickle(file_path_name)
        print(f" Save  daily data: {len(df)}  to  pickle: {file_path_name}")
    else:
        print(f" Daily data in <{file_path_name}> is new , Needn't update ! ")
        

if  __name__ == "__main__":

    #read_daily_counts_from_pickle(table_name="swl_day") 
    #df = read_daily_counts_from_pickle(table_name="stock_day_adj_fillna")
    #print(df[df.ts_code=='600519.SH'])
    #is_daily_pickle_new(table_name="stock_day_adj_fillna", N=250)
    #is_daily_pickle_new(table_name='swl_day')
    save_daily_count_to_pickle(table_name='swl_day')
    #save_daily_count_to_pickle(table_name="stock_day_adj_fillna")
    
    
    


# coding:utf-8
import os
"""创建本地文件夹
1. setting_path ==> 用于存放配置文件 setting.cfg
2. cache_path ==> 用于存放临时文件
3. log_path ==> 用于存放储存的log
"""

path = os.path.expanduser('~')
rq_path_name = 'rrsdk'
rq_path = f'{path}/.{rq_path_name}'
#print(rq_path)


def generate_path(name):
    return '{}{}{}'.format(rq_path, os.sep, name)


def make_dir(path, exist_ok=True):
    os.makedirs(path, exist_ok=exist_ok)
    print(f"makedir path: {path}")


setting_path = generate_path('setting')
cache_path = generate_path('cache')
log_path = generate_path('log')


def make_dir_path():
    make_dir(rq_path, exist_ok=True)
    make_dir(setting_path, exist_ok=True)
    make_dir(cache_path, exist_ok=True)
    make_dir(log_path, exist_ok=True)
  
   
if __name__ == '__main__':
    make_dir_path()



# coding:utf-8
import os
"""创建本地文件夹
1. setting_path ==> 用于存放配置文件 setting.cfg
2. cache_path ==> 用于存放临时文件
3. log_path ==> 用于存放储存的log
4. download_path ==> 下载的数据/财务文件
5. strategy_path ==> 存放策略模板
6. bin_path ==> 存放一些交易的sdk/bin文件等
"""

path = os.path.expanduser('~')
rq_path = '{}{}{}'.format(path, os.sep, '.rrshare')

def generate_path(name):
    return '{}{}{}'.format(rq_path, os.sep, name)


def make_dir(path, exist_ok=True):
    os.makedirs(path, exist_ok=exist_ok)
    #print(f"makedir path: {path}")


setting_path = generate_path('setting')
cache_path = generate_path('cache')
log_path = generate_path('log')
#download_path = generate_path('downloads')
#strategy_path = generate_path('strategy')
#bin_path = generate_path('bin')  #给一些dll文件存储用

def make_dir_path():
    make_dir(rq_path, exist_ok=True)
    make_dir(setting_path, exist_ok=True)
    make_dir(cache_path, exist_ok=True)
    #make_dir(download_path, exist_ok=True)
    make_dir(log_path, exist_ok=True)
    #make_dir(strategy_path, exist_ok=True)
    #make_dir(bin_path, exist_ok=True)

make_dir_path()



from __future__ import print_function
from subprocess import Popen, PIPE, STDOUT

# Globals ######################################################################
DEBUG = False


def execute(command, shell=None, working_dir=".", echo=False, echo_indent=0):
    """Execute a command on the command-line.
    :param str,list command: The command to run
    :param bool shell: Whether or not to use the shell.  This is optional; if
        ``command`` is a basestring, shell will be set to True, otherwise it will
        be false.  You can override this behavior by setting this parameter
        directly.
    :param str working_dir: The directory in which to run the command.
    :param bool echo: Whether or not to print the output from the command to
        stdout.
    :param int echo_indent: Any number of spaces to indent the echo for clarity
    :returns: tuple: (return code, stdout)
    Example
        >>> from executor import execute
        >>> return_code, text = execute("dir")
    """
    if shell is None:
        shell = True if isinstance(command, str) else False

    p = Popen(command, stdin=PIPE, stdout=PIPE,
              stderr=STDOUT, shell=shell, cwd=working_dir)

    if echo:
        stdout = ""
        while p.poll() is None:
            # This blocks until it receives a newline.
            line = p.stdout.readline()
            print(" " * echo_indent, line, end="")
            stdout += line

        # Read any last bits
        line = p.stdout.read()
        print(" " * echo_indent, line, end="")
        print()
        stdout += line
    else:
        stdout, _ = p.communicate()

    return (p.returncode, stdout)


from rrshare.rqSU import save_swl1_realtime
from rrshare.rqFactor import  swl1_index_realtime_prs


def record_swl1_index_realtime():
    save_swl1_realtime()
    swl1_index_realtime_prs()


if __name__  == '__main__':
    record_swl1_index_realtime()



from rrshare.rqFactor.stock_RS_OH_MA import update_stock_PRS_new
from rrshare.rqFactor.stock_RS_OH_MA import update_stock_PRS_day

def record_stock_PRS():
    try:
        update_stock_PRS_day()
    except Exception as e:
        print(e)

def record_stock_PRS_new():
    try:
        update_stock_PRS_new()
    except Exception as e:
        print(e)

if __name__ == '__main__':
    record_stock_PRS()
    record_stock_PRS_new()




#!/usr/bin/python
import psycopg2
from rrshare.rqUtil.config_postgresql import database_uri, get_config_ini

def connect_postgrsql():
    """ Connect to the PostgreSQL database server """
    conn = None
    try:
        # read connection parameters
        params = database_uri
        # connect to the PostgreSQL server
        print('Connecting to the PostgreSQL database...')
        conn = psycopg2.connect(**params)
        print(conn)
        # create a cursor
        cur = conn.cursor()
    	# execute a statement
        print('PostgreSQL database version:')
        cur.execute('SELECT version()')
        # display the PostgreSQL database server version
        db_version = cur.fetchone()
        print(db_version)
	    # close the communication with the PostgreSQL
        cur.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()
            print('Database connection closed.')


def client_pgsql(db_name="rrshare"):
    """ Connect to the PostgreSQL database server """
    conn = None
    try:
        # read connection parameters
        section_name = f"pgsql_{db_name}"
        params = get_config_ini(filename='config.ini', section=section_name)
        # connect to the PostgreSQL server
        print('Connecting to the PostgreSQL database...')
        conn = psycopg2.connect(**params)
        print(conn)
        # create a cursor
        cur = conn.cursor()
    	# execute a statement
        print('PostgreSQL database version:')
        cur.execute('SELECT version()')
        # display the PostgreSQL database server version
        db_version = cur.fetchone()
        print(db_version)
	    # close the communication with the PostgreSQL
        cur.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()
            print('Database connection closed.')


if __name__ == '__main__':
    connect_postgrsql()
    client_pgsql()


import csv
import json

import numpy as np
import pandas as pd


def rq_util_to_json_from_pandas(data):
    """
    explanation:
        将pandas数据转换成json格式		
    params:
        * data ->:
            meaning: pandas数据
            type: null
            optional: [null]
    return:
        dict
    demonstrate:
        Not described
    output:
        Not described
    """

    """需要对于datetime 和date 进行转换, 以免直接被变成了时间戳"""
    if 'datetime' in data.columns:
        data.datetime = data.datetime.apply(str)
    if 'date' in data.columns:
        data.date = data.date.apply(str)
    return json.loads(data.to_json(orient='records'))


def rq_util_to_json_from_numpy(data):
    pass


def rq_util_to_json_from_list(data):
    pass


def rq_util_to_list_from_pandas(data):
    """
    explanation:
         将pandas数据转换成列表		
    params:
        * data ->:
            meaning: pandas数据
            type: null
            optional: [null]
    return:
        list
    demonstrate:
        Not described
    output:
        Not described
    """

    return np.asarray(data).tolist()


def rq_util_to_list_from_numpy(data):
    """
    explanation:
        将numpy数据转换为列表		
    params:
        * data ->:
            meaning: numpy数据
            type: null
            optional: [null]
    return:
        None
    demonstrate:
        Not described
    output:
        Not described
    """

    return data.tolist()


def rq_util_to_pandas_from_json(data):
    """
    explanation:
        将json数据载入为pandas数据		
    params:
        * data ->:
            meaning: json数据
            type: null
            optional: [null]
    return:
        DataFrame
    demonstrate:
        Not described
    output:
        Not described
    """
    if isinstance(data, dict):
        return pd.DataFrame(data=[data, ])
    else:
        return pd.DataFrame(data=[{'value': data}])


def rq_util_to_pandas_from_list(data):
    """
    explanation:
        将列表数据转换为pandas	
    params:
        * data ->:
            meaning: 列表数据
            type: list
            optional: [null]
    return:
        DataFrame
    demonstrate:
        Not described
    output:
        Not described
    """

    if isinstance(data, list):
        return pd.DataFrame(data=data)



#!/usr/bin/python
from configparser import ConfigParser

from rrshare.rqUtil.rqLogs import rq_util_log_info
from rrshare.rqUtil.config_setting import get_path_file_name

def get_config_ini(section='rrshare',filename='config.ini'):
    # create a parser
    parser = ConfigParser()
    # read config file
    parser.read(get_path_file_name(path_name='setting',file_name=filename))
    # get section, default to postgresql
    db = {}
    if parser.has_section(section):
        params = parser.items(section)
        for param in params:
            db[param[0]] = param[1]
    else:
        raise Exception('Section {0} not found in the {1} file'.format(section, filename))
    #rq_util_log_info(db)
    return db

database_uri = get_config_ini('pgsql_rrshare')
rq_util_log_info(database_uri)


from rrshare.rqUtil import rq_util_get_pre_trade_date

from rrshare.rqSU import  rq_save_swl_day_pg
                

def record_swl_day(N=265):
    """ swl_day last 10 days data, N +10"""
    #rq_save_swl_industry_list_stock_belong()
    rq_save_swl_day_pg(rq_util_get_pre_trade_date(n=N))

if __name__ == '__main__':
    #print(rq_util_get_pre_trade_date(n=255))
    record_swl_day(265)


from rrshare.rqUpdate.update_stock_day import record_stock_day
from rrshare.rqUpdate.update_swl_day import record_swl_day
from rrshare.rqUpdate.update_stock_RS_OH_MA import record_stock_PRS, record_stock_PRS_new



def main_record():
    record_stock_day()
    record_swl_day()

    record_stock_PRS()
    record_stock_PRS_new()



if __name__ == '__main__':
    main_record()



# -*- coding: utf-8 -*-
import time
import pandas as pd
import psycopg2
from sqlalchemy import create_engine
import logging
from rrshare.rqUtil.rqSingleton import  ConSqlDb
from rrshare.rqUtil.rqLogs import (rq_util_log_debug, rq_util_log_expection,
                                             rq_util_log_info)

from rrshare.rqUtil import (rq_util_get_trade_range, rq_util_get_last_tradedate, 
                        rq_util_get_pre_trade_date)
from rrshare.rqUtil.rqSetting import  setting

logging.basicConfig(level=logging.INFO, format=' %(asctime)s- %(levelname)s-%(message)s')
password_pgsql = setting['PGSQL_PASSWORD']


class PgsqlClass(ConSqlDb):
         
    def __init__(self,
            database ='postgres',
            user='postgres', 
            password=password_pgsql,
            host='39.108.68.163',
            port='5432'):

        self.database = database
        self.user =  user
        self.password = password
        self.host = host
        self.port = port

        #self.engine = create_engine(f'postgresql+psycopg2://{self.user}:{self.password}@{self.host}:{self.port}/{self.database}')
        #print(self.engine)
        self.conn = psycopg2.connect(user=self.user,
                password=self.password,
                host=self.host,
                port=self.port,
                database = database
        )
        #print(self.conn)
    
    def client_pg(self, db_name):
        client = create_engine(f'postgresql+psycopg2://{self.user}:{self.password}@{self.host}:{self.port}/{db_name}')
        #print(client)
        return client


    def create_psqlDB(self,db_name):
        try:
            conn = self.conn
            #conn = psycopg2.connect(user=self.user,password=self.password,host=self.host, port=self.port)
            print(conn)
            psql = f'''CREATE DATABASE {db_name}'''
            #OWNER {self.user};
            #        GRANT ALL PRIVILEGES ON DATABASE {db_name}  to {self.user};
            #        '''
            conn.autocommit = True
            cursor = conn.cursor()
            cursor.execute(psql)
            #conn.commit()
            print(f'database {db_name} created successfully use by user  {self.user}')
            #cursor.close()
            conn.close()
        except (Exception, psycopg2.DatabaseError) as error:
            print('Error while create PostgresSQL database', error)

   
    def create_table(self, db_name, table_name, tsql):
        try:
            conn = psycopg2.connect(user=self.user,password=self.password,host=self.host, port=self.port,database=db_name) 
            cursor = conn.cursor()
            conn.autocommit = True
            cursor.execute(f"DROP TABLE if EXISTS {table_name}")
            cursor.execute(tsql)
            #cursor.commit()
            print(f'{table_name} created  successfully on database {db_name}')
            conn.close()
        except Exception as e:
            print(e)


    def insert_to_psql(self, df,db_name, table_name, if_exists='append'):
        try:
            engine = create_engine(f'postgresql+psycopg2://{self.user}:{self.password}@{self.host}:{self.port}/{db_name}')
            df.to_sql(name=table_name, con=engine, index=False, if_exists=if_exists)
            print(f'write to database:{db_name} table:{table_name} ok !')
        except Exception as e:
            print(e)

    def read_psql(self, db_name, table_name):
        engine = create_engine(f'postgresql+psycopg2://{self.user}:{self.password}@{self.host}:{self.port}/{db_name}')
        return pd.read_sql_table(table_name,engine)
       
def client_pgsql(database='rrshare'):
    try:
        return PgsqlClass().client_pg(db_name=database)
    except Exception as e:
        print(e)     

def save_data_to_postgresql(data,table_name,client=client_pgsql(),if_exists='replace'):
        data.to_sql(table_name,client,index=False,if_exists=if_exists)
    
def load_data_from_postgresql(mes='',client=client_pgsql()):
    res=pd.read_sql(mes,client)
    return res

def read_data_from_pg(table_name='', client=client_pgsql()):
    ''' load all data of table_name
    '''
    res = pd.read_sql_table(table_name, client)
    return res


def read_table_from_pg(table_name='',period=1, client=None):
    '''only select trade_date, can't select columns
        default out lasttradedate data
        period -->int >= 1
    '''
    try: 
        start_date = rq_util_get_pre_trade_date(rq_util_get_last_tradedate(), period-1)
        logging.info(start_date)
        sql = ''.join(['SELECT * FROM ', f'{table_name}', f" WHERE trade_date >= date_trunc('day', timestamp '"+start_date+"') order by trade_date ASC;"])
        logging.info(sql)
        res = pd.read_sql(sql,client)
        logging.info(f'\n {res}')
        return res
    except Exception as e:
        logging.info(e)


def read_sql_from_pg(start_date=None,data=None,table_name=None,client=client_pgsql()):
    '''can select columns and trade_date
    '''
    sql = ''.join(['SELECT ', data, ' FROM ', f'{table_name}', f" WHERE trade_date >= date_trunc('day', timestamp '"+start_date+"') order by trade_date ASC;"])
    #sql= 'select ' + data + ' from ' +table_name+" where trade_date >= date_trunc('day',timestamp '"+start_date+"') order by trade_date ASC;"
    try:
        t=time.time()
        res=pd.read_sql(sql,client)
        t1=time.time()
        tt = t1-t
        logging.info(f'read  data from {table_name} ,take {tt}')
        return res
    except Exception as e:
        logging.error(e)



def read_unique_data_from_pg(cols='*',table_name='',period=1, client=client_pgsql()):
    ''' select trade_date and columns( * is all), data GROUP BY trade_date and _ts_code --> duplicates 
    '''
    try: 
        start_date = rq_util_get_pre_trade_date(rq_util_get_last_tradedate(), period-1)
        logging.info(start_date)
        sql = f"""
        SELECT {cols} 
        FROM  {table_name}
        GROUP BY trade_date, ts_code 
        """        
        logging.info(sql)
        res = pd.read_sql(sql,client)
        logging.info(f'\n {res}')
        return res
    except Exception as e:
        logging.info(e)



if __name__ == '__main__':
    import os
    psql = PgsqlClass()
    """
    psql.create_psqlDB(db_name='rrshare')
    path = '/home/rome/rrshare/rrshare/sql'
    file_n = 'stock_day_fillna.sql'
    file_path = os.path.join(path, file_n)
    print(file_path)
    with open(file_path, 'r') as f:
        tsql = f.read()
    #print(tsql)
    psql.create_table('rrshare','stock_day_fillna',tsql)
    """
    psql2 = PgsqlClass()
    print(id(psql), id(psql2))
    psql.client_pg(db_name='rrshare')
    
    print(read_data_from_pg('swl_list').head())
    print(read_table_from_pg(table_name='stock_day_adj_fillna',client=client_pgsql('rrshare')))
    pass



# coding:utf-8

# singleton
from rrshare.rqUtil.rqSingleton import (Singleton, Singleton_wraps, ConSqlDb)
# config
from rrshare.rqUtil.config_setting import setting

# code function
from rrshare.rqUtil.rqCode import (rq_util_code_tolist, rq_util_code_tostr, rq_util_code_tosrccode, rq_util_code_adjust_ctp)
# csv
from rrshare.rqUtil.rqCsv import rq_util_save_csv

# trade_time
from rrshare.rqUtil.rqTrade_time import is_trade_time_secs_cn, is_before_tradetime_secs_cn, is_tradeday_and_market_opened

# date
from rrshare.rqUtil.rqDate import (rq_util_calc_time, rq_util_date_int2str,
                                     rq_util_date_stamp, rq_util_date_str2int,
                                     rq_util_date_today, rq_util_date_valid,
                                     rq_util_datetime_to_strdate,
                                     rq_util_stamp2datetime,
                                     rq_util_get_date_index,
                                     rq_util_tdxtimestamp,
                                     rq_util_get_index_date, rq_util_id2date,
                                     rq_util_is_trade, rq_util_ms_stamp,
                                     rq_util_realtime, rq_util_select_hours,
                                     rq_util_select_min, rq_util_time_delay,
                                     rq_util_time_now, rq_util_time_stamp,
                                     rq_util_to_datetime, rq_util_today_str,
                                     rqTZInfo_CN)
# trade date
from rrshare.rqUtil.rqDate_trade import (rq_util_date_gap,
                                           rq_util_format_date2str,
                                           rq_util_future_to_realdatetime,
                                           rq_util_future_to_tradedatetime,
                                           rq_util_get_last_datetime,
                                           rq_util_get_last_day,
                                           rq_util_get_last_tradedate,
                                           rq_util_get_next_datetime,
                                           rq_util_get_next_day,
                                           rq_util_get_next_trade_date,
                                           rq_util_get_next_period,
                                           rq_util_get_order_datetime,
                                           rq_util_get_pre_trade_date,
                                           rq_util_get_real_date,
                                           rq_util_get_real_datelist,
                                           rq_util_get_trade_datetime,
                                           rq_util_get_trade_gap,
                                           rq_util_get_trade_range,
                                           rq_util_if_trade,
                                           rq_util_if_tradetime,
                                           rq_util_future_to_realdatetime,
                                           rq_util_future_to_tradedatetime,
                                           trade_date_sse)

# log
from rrshare.rqUtil.rqLogs import (rq_util_log_debug, rq_util_log_expection,
                                     rq_util_log_info)

# postgres
from rrshare.rqUtil.rqPgsql import (PgsqlClass,client_pgsql, read_data_from_pg,read_sql_from_pg,read_table_from_pg,read_unique_data_from_pg,save_data_to_postgresql)

# Parameter
from rrshare.rqUtil.rqParameter import (
    PERIODS, SWL_LEVEL, startDate, UPDATETIME,   
    ACCOUNT_EVENT, AMOUNT_MODEL, BROKER_EVENT, BROKER_TYPE, DATASOURCE,
    ENGINE_EVENT, EVENT_TYPE, EXCHANGE_ID, FREQUENCE, MARKET_ERROR,
    MARKET_EVENT, MARKET_TYPE, ORDER_DIRECTION, ORDER_EVENT, ORDER_MODEL,
    TIME_CONDITION, VOLUME_CONDITION,
    ORDER_STATUS, OUTPUT_FORMAT, RUNNING_ENVIRONMENT, TRADE_STATUS, RUNNING_STATUS)

# format
from rrshare.rqUtil.rqTransform import (rq_util_to_json_from_pandas,
                                          rq_util_to_list_from_numpy,
                                          rq_util_to_list_from_pandas,
                                          rq_util_to_pandas_from_json,
                                          rq_util_to_pandas_from_list)



from rrshare.rqFactor.swl_RS_OH_MA import update_swl_PRS_day, update_swl_PRS_new

def record_swl_PRS():
    try:
        update_swl_PRS_day()
    except Exception as e:
        print(e)

def record_swl_PRS_new():
    try:
        update_swl_PRS_new()
    except Exception as e:
        print(e)

if __name__ == '__main__':
    record_swl_PRS()
    record_swl_PRS_new()




from rrshare.rqUtil import PgsqlClass
from rrshare.rqFetch import sw_index_class_L1_L2, sw_index_class


def save_sw_index_list():
    """ """
    for level in ["L1","L2"]:
        PgsqlClass().insert_to_psql(sw_index_class(level), 'rrshare',f'swl_list_{level}',if_exists='replace')
        
    PgsqlClass().insert_to_psql(sw_index_class_L1_L2(), 'rrshare','swl_list',if_exists='replace')


if __name__ == '__main__':
    save_sw_index_list()
   


import csv


def rq_util_save_csv(data, name, column=None, location=None):
    # 重写了一下保存的模式
    # 增加了对于可迭代对象的判断 2017/8/10
    """
    explanation:
        将数据保存为csv
    
    params:
        * data->
            含义: 需要被保存的数据
            类型: List
            参数支持: []
        * name:
            含义: 要保存的名字
            类型: str
            参数支持: []
        * column:
            含义: 行的名称(可选)
            类型: str
            参数支持: [None]
        * location:
            含义: 保存位置(可选)
            类型: str
            参数支持: []
    """
    assert isinstance(data, list)
    if location is None:
        path = './' + str(name) + '.csv'
    else:
        path = location + str(name) + '.csv'
    with open(path, 'w', newline='') as f:
        csvwriter = csv.writer(f)
        if column is None:
            pass
        else:
            csvwriter.writerow(column)

        for item in data:

            if isinstance(item, list):
                csvwriter.writerow(item)
            else:
                csvwriter.writerow([item])


if __name__ == '__main__':
    rq_util_save_csv(['a', 'v', 2, 3], 'test')
    rq_util_save_csv([['a', 'v', 2, 3]], 'test2')



# coding:utf-8
import sqlite3
import os

CACHE = None


def _init(cache_file):
    """Creates a new Cache object."""
    global CACHE
    CACHE = Cache(cache_file)


def get_cache(config_file=None):
    """Used to retrieve the global cache object."""
    if CACHE is None:
        _init(config_file)
    return CACHE


class Cache():
    """This object is used to interface with the job cache.  It uses a SQLite3
    database to store the information.
    :param str cache_file: The path to the cache file.  This will be created if
        it does not already exist.
    """
    def __init__(self, cache_file):
        self.filename = cache_file

        if not os.path.isfile(self.filename):
            self._create(self.filename)

        self.conn = sqlite3.connect(self.filename)
        self.cur = self.conn.cursor()
        self.cur.execute("PRAGMA foreign_keys = ON")

    def __del__(self):
        """Commit the changes and close the connection."""
        if getattr(self, "conn", None):
            self.conn.commit()
            self.conn.close()

    def _create(self, cache_file):
        """Create the tables needed to store the information."""
        conn = sqlite3.connect(cache_file)
        cur = conn.cursor()
        cur.execute("PRAGMA foreign_keys = ON")
        cur.execute('''
            CREATE TABLE jobs(
                hash TEXT NOT NULL UNIQUE PRIMARY KEY, description TEXT NOT NULL,
                last_run REAL, next_run REAL, last_run_result INTEGER)''')

        cur.execute('''
            CREATE TABLE history(
                hash TEXT, description TEXT, time REAL, result INTEGER,
                FOREIGN KEY(hash) REFERENCES jobs(hash))''')

        conn.commit()
        conn.close()

    def has(self, job):
        """Checks to see whether or not a job exists in the table.
        :param dict job: The job dictionary
        :returns: True if the job exists, False otherwise
        """
        return bool(self.cur.execute('SELECT count(*) FROM jobs WHERE hash=?', (job["id"],)))

    def get(self, id):
        """Retrieves the job with the selected ID.
        :param str id: The ID of the job
        :returns: The dictionary of the job if found, None otherwise
        """
        self.cur.execute("SELECT * FROM jobs WHERE hash=?", (id,))
        item = self.cur.fetchone()
        if item:
            return dict(zip(
                ("id", "description", "last-run", "next-run", "last-run-result"),
                item))
        return None


    def update(self, job):
        """Update last_run, next_run, and last_run_result for an existing job.
        :param dict job: The job dictionary
        :returns: True
        """
        self.cur.execute('''UPDATE jobs
            SET last_run=?,next_run=?,last_run_result=? WHERE hash=?''', (
            job["last-run"], job["next-run"], job["last-run-result"], job["id"]))

    def add_job(self, job):
        """Adds a new job into the cache.
        :param dict job: The job dictionary
        :returns: True
        """
        self.cur.execute("INSERT INTO jobs VALUES(?,?,?,?,?)", (
            job["id"], job["description"], job["last-run"], job["next-run"], job["last-run-result"]))
        return True

    def add_result(self, job):
        """Adds a job run result to the history table.
        :param dict job: The job dictionary
        :returns: True
        """
        self.cur.execute(
            "INSERT INTO history VALUES(?,?,?,?)",
            (job["id"], job["description"], job["last-run"], job["last-run-result"]))
        return True